// ****************************************************************************
// copyright (c) 2000,2001 Horst Knorr <hk_classes@knorrnet.de>
// This file is part of the hk_classes library.
// This file may be distributed and/or modified under the terms of the
// GNU Library Public License version 2 as published by the Free Software
// Foundation and appearing in the file COPYING included in the
// packaging of this file.
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
// ****************************************************************************
#include "hk_kdebutton.h"
#include "hk_kdeform.h"
#include "hk_kdequery.h"
#include "hk_kdetable.h"
#include <hk_database.h>
#include <qwidget.h>
#include <qapplication.h>
#include <X11/X.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>


hk_kdebutton::hk_kdebutton(QWidget* wid,hk_form* form)
	:QPushButton(wid),
	hk_button(form)
{
#ifdef HK_DEBUG
hkdebug("hk_kdebutton::hk_kdebutton");
#endif
 QObject::connect(this,SIGNAL(clicked()),this,SLOT(button_clicked()));
}

hk_kdebutton::~hk_kdebutton()
{
#ifdef HK_DEBUG
hkdebug("hk_kdebutton::~hk_kdebutton");
#endif

}

bool hk_kdebutton::widget_specific_coordinates(uint px,uint py,uint pwidth,uint pheight)
{
	blockSignals(true);
	setGeometry(px,py,pwidth,pheight);
	blockSignals(false);
 return true;
}

bool hk_kdebutton::widget_specific_open_form(void)
{
#ifdef HK_DEBUG
hkdebug("hk_kdebutton::widget_specific_open_form");
#endif
   hk_kdeform* w=find_existing_form();

   if (w==NULL)  // not open yet
   {
   	w=new hk_kdeform(0,0,WDestructiveClose);
   	if (p_form!=NULL)w->set_database(p_form->database());
   	else w->set_database(database());
   	w->load_form(object());
   	w->set_viewmode();
   	if (show_maximized())w->showMaximized();
	else w->show();
   }
   else
   {
	if (w->isMinimized())
	{
	   w->showMaximized();
	}
        w->setActiveWindow();
   	w->raise();
   }
return true;
}

bool hk_kdebutton::widget_specific_close_form(void)
{
#ifdef HK_DEBUG
hkdebug("hk_kdebutton::widget_specific_close_form");
#endif
   hk_kdeform* w=find_existing_form();
   if (w==NULL)	return false;

   w->close();
   return true;
}

void hk_kdebutton::button_clicked(void)
{
#ifdef HK_DEBUG
hkdebug("hk_kdebutton::button_clicked");
#endif
   push_action();
}

void	hk_kdebutton::widget_specific_label_changed(void)
{
#ifdef HK_DEBUG
hkdebug("hk_kdebutton::widget_specific_label_changed");
#endif
   setText(label().c_str());
}


void hk_kdebutton::resizeEvent ( QResizeEvent * event )
{
#ifdef HK_DEBUG
hkdebug("hk_kdebutton::resizeEvent");
#endif
  QPushButton::resizeEvent(event);
  reposition();

}


void  hk_kdebutton::moveEvent ( QMoveEvent * event)
{
#ifdef HK_DEBUG
hkdebug("kdebutton::resizeEvent");
#endif
    QPushButton::moveEvent(event);
    reposition();
}

void hk_kdebutton::reposition(void)
{
#ifdef HK_DEBUG
hkdebug("kdebutton::reposition");
#endif
  if (p_form==NULL ||p_setcoordinates==true) return;
  p_setwidgetcoordinates=true;
    unsigned int ppx=p_form->horizontal2relativ(QWidget::x());
    unsigned int ppy=p_form->vertical2relativ(QWidget::y());
    unsigned int pph=p_form->vertical2relativ(QWidget::height(),p_dynamicheight);
    unsigned int ppw=p_form->horizontal2relativ(QWidget::width(),p_dynamicwidth);
    set_size(ppx,ppy,ppw,pph,p_dynamicwidth,p_dynamicheight);
  p_setwidgetcoordinates=false;
}

void hk_kdebutton::widget_specific_font_changed(void)
{
#ifdef HK_DEBUG
hkdebug("hk_kdebutton::widget_specific_font_changed");
#endif
    setFont(QFont(p_font.c_str(),p_fontsizeint));
}

hk_kdeform* hk_kdebutton::find_existing_form(void)
{
  hk_kdeform* w = NULL;
      if (p_form->database()!=NULL)     // is the form already open?
      {
       QWidget* sf=dynamic_cast<QWidget*>
       			(p_form->database()->existing_presentation(object(),hk_presentation::form));
	while (sf!=NULL && w==NULL)
	{
	  sf=sf->parentWidget();
	  w= dynamic_cast <hk_kdeform*> (sf);
	}
      }
 return w;
}


bool hk_kdebutton::widget_specific_open_table(void)
{
    if (p_form==NULL) return false;
    if (p_form->database()!=NULL)
    {
      	hk_kdetable* w=new hk_kdetable();
   	hk_datasource* d=p_form->database()->new_table(object());
   	w->set_datasource(d);
   	w->set_viewmode();
   	if (show_maximized())w->showMaximized();
	else w->show();
    }
    return true;
}

bool hk_kdebutton::widget_specific_open_query(void)
{
    if (p_form==NULL) return false;
    if (p_form->database()!=NULL)
    {
      	hk_kdequery* w=new hk_kdequery();
   	hk_datasource* d=p_form->database()->new_resultquery();
   	d->set_name(object());
   	w->set_datasource(d);
   	w->load_query();
   	w->set_viewmode();
   	if (show_maximized())w->showMaximized();
	else w->show();
    }
    return true;
}

