// ****************************************************************************
// copyright (c) 2000,2001 Horst Knorr <hk_classes@knorrnet.de>
// This file is part of the hk_kdeclasses library.
// This file may be distributed and/or modified under the terms of the
// GNU General Public License version 2 as published by the Free Software
// Foundation and appearing in the file COPYING included in the
// packaging of this file.
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
// ****************************************************************************
//$Revision: 1.6 $

#include "hk_kdecsvimportdialog.h"
#include "hk_kdecsvimportdialog.moc"

#include <qcheckbox.h>
#include <qcombobox.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qtoolbutton.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qfiledialog.h>
#include <klocale.h>

/*
 *  Constructs a hk_kdecsvimportdialog which is a child of 'parent', with the
 *  name 'name' and widget flags set to 'f'
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
 */
hk_kdecsvimportdialog::hk_kdecsvimportdialog( QWidget* parent,  const char* name, bool modal, WFlags fl )
    : hk_kdecsvimportdialogbase( parent, name, modal, fl ),hk_importcsv()
{
    if ( !name )
	setName( "hk_kdecsvimportdialog" );
    setCaption( hk_translate( "CSV Import Dialog"  ).c_str() );
    TextLabel1->setText( hk_translate( "Filename:"  ).c_str() );
    TextLabel3->setText( hk_translate( "Text delimiter"  ).c_str() );
    TextLabel1_3->setText( hk_translate( "Field names in first row"  ).c_str() );
    textdelimiterfield->setText( "\""   );
    TextLabel1_2->setText( hk_translate("Tablename:"  ).c_str() );
    TextLabel2->setText( hk_translate("Column Separator"  ).c_str() );
    p_tab=hk_translate("Tab");
    columnseparatorfield->insertItem(  ","  );
    columnseparatorfield->insertItem(  ";"  );
    columnseparatorfield->insertItem( p_tab.c_str());
    buttonOk->setText( hk_translate( "&OK"  ).c_str() );
    buttonCancel->setText( hk_translate( "&Cancel"  ).c_str() );
    buttonHelp->setText( hk_translate( "&Help"  ).c_str() );


    connect( buttonOk, SIGNAL( clicked() ), this, SLOT( ok_clicked() ) );
    connect( buttonCancel, SIGNAL( clicked() ), this, SLOT( reject() ) );
    connect( filebutton, SIGNAL( clicked() ), this, SLOT( filebutton_clicked() ) );
    connect( filefield, SIGNAL( textChanged(const QString&) ), this, SLOT( buttons_enabled() ) );
    connect( tablename, SIGNAL( textChanged(const QString&) ), this, SLOT( buttons_enabled() ) );
    connect( columnseparatorfield, SIGNAL( textChanged(const QString&) ), this, SLOT( buttons_enabled() ) );

}

hk_kdecsvimportdialog::~hk_kdecsvimportdialog()
{
    // no need to delete child widgets, Qt does it all for us
//    delete p_import;
}

void hk_kdecsvimportdialog::ok_clicked(void)
{
//  wanna_debug(true);
  set_filename(filefield->text().latin1());
  datasource()->set_name(tablename->text().latin1());
  set_firstrow_contains_fieldnames(firstrow->isChecked());
  hk_string sep=columnseparatorfield->currentText().latin1();
  if (sep==p_tab) sep="\t";
  set_filedefinition(textdelimiterfield->text().latin1(),sep.c_str());
  if ( execute()) accept();
}


void hk_kdecsvimportdialog::filebutton_clicked()
{

 p_file = QFileDialog::getOpenFileName( QString::null, QString::null, this,"","Load CSV files" );
 filefield->setText(p_file);
}

void hk_kdecsvimportdialog::buttons_enabled()
{
  if (    !filefield->text().isEmpty()
        &&!columnseparatorfield->currentText().isEmpty()
	&&!tablename->text().isEmpty()
      )
       buttonOk->setEnabled(true);
  else buttonOk->setEnabled(false);
}

