// ****************************************************************************
// copyright (c) 2000,2001 Horst Knorr <hk_classes@knorrnet.de>
// This file is part of the hk_kdeclasses library.
// This file may be distributed and/or modified under the terms of the
// GNU General Public License version 2 as published by the Free Software
// Foundation and appearing in the file COPYING included in the
// packaging of this file.
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
// ****************************************************************************

#ifndef HK_KDEform_H
#define HK_KDEform_H
#include <hk_form.h>
#include <qvariant.h>
#include <qwidget.h>
#include <hk_presentation.h>
#include <fstream.h>
#include <hk_class.h>

class QVBoxLayout;
class QHBoxLayout;
class QGridLayout;
class QLabel;
class QScrollView;
class hk_kdesimpleform;
class QToolButton;
class QButtonGroup;
class hk_kderowselector;
class hk_kdetoolbar;

/**
*
*@short  KDE high level widget to display and design forms
*@author Horst Knorr (hk_classes@knorrnet.de)
*@version $Revision: 1.13 $
*
*this is a high level widget to display and design forms. It hereby uses @ref hk_kdesimpleform.
*To use it set the database with @ref set_database. If you want to show or edit an existing form
* load it with @ref load_form.
*/

class hk_kdeform : public QWidget,public hk_class
{
    Q_OBJECT

public:
    hk_kdeform( QWidget* parent = 0, const char* name = 0, WFlags fl = 0 );
virtual    ~hk_kdeform();

    	hk_presentation::enum_mode	mode(void);
/**
* See @ref hk_kdesimpleform for details
*/
	void 			set_mode(hk_presentation::enum_mode s);
	void  			set_viewmode(void);
	void			set_designmode(void);
/**
* See @ref hk_kdesimpleform for details
*/
 	hk_dsgrid*		new_grid(void);
/**
* See @ref hk_kdesimpleform for details
*/
	hk_dslineedit*	 	new_lineedit(void);
/**
* See @ref hk_kdesimpleform for details
*/
	hk_dsboolean*	 	new_bool(void);
/**
* See @ref hk_kdesimpleform for details
*/
	hk_button* 		new_button(void);
/**
* See @ref hk_kdesimpleform for details
*/
	hk_dsrowselector*	new_rowselector(void);
/**
* See @ref hk_kdesimpleform for details
*/
	hk_dsmemo*		new_memo(void);
/**
* See @ref hk_kdesimpleform for details
*/
	void			set_formsize(unsigned int width, unsigned int height);
/**
* See @ref hk_kdesimpleform for details
*/
	void			set_database(hk_database* db);
	hk_database* 		database(void);
	list<hk_datasource*>*	datasources(void);
 	long			new_datasource(const hk_string& name,bool query=false);
	hk_datasource* 		get_datasource(long nr);
/**
* returns a pointer to the internally used hk_kdesimpleform. Don't delete the object!
*/
	hk_kdesimpleform*	simpleform(void);
/**
* if r=true the simpleform will also be set. Usually you have to leave it to "true"
*/
	bool		set_presentationdatasource(long n,bool r=true);
	long  		presentationdatasource(void);
	void		loaddata(const hk_string& definition);
	void 		savedata(ofstream& s);
	void		load_form(const hk_string& n);
/**
*when the datasource vanishes (i.e. will be destroyed) and autoclose is true, the hk_kdetable will be
*closed otherwise it still is on your screen.
*/

void set_autoclose(bool c);
/**
*if the designwindow should not be shown set this true. Default is false.
*/
void set_nodesignmode(bool n);

protected:
virtual void closeEvent ( QCloseEvent* e);
virtual void 		mousePressEvent(QMouseEvent*);
protected slots:
	void formbutton_clicked(void);
	void designbutton_clicked(void);
	void fieldbutton_clicked(void);
	void field_created(void);
	void save_form(void);
private:
    hk_kdetoolbar* toolbar;
    QButtonGroup* formbuttongroup;
    QToolButton* formpointerbutton;
    QToolButton* formlineeditbutton;
    QToolButton* formlabelbutton;
    QToolButton* formmultilineeditbutton;
    QToolButton* formbooleanbutton;
    QToolButton* formcomboboxbutton;
    QToolButton* formtablebutton;
    QToolButton* formselectorbutton;
    QToolButton* formbuttonbutton;
    hk_kdesimpleform* p_form;
    hk_kderowselector* p_rowselector;
//    QButtonGroup* ButtonGroup6;
    QLabel* infolabel;
    QVBoxLayout* hk_kdeformLayout;
//    QHBoxLayout* toolbarLayout;
    QHBoxLayout* Layoutgrid_formtool;
    QVBoxLayout* formbuttongroupLayout;
    QScrollView* p_scrollview;

    QHBoxLayout* Layout3;
    bool p_nodesignmode;
};

#endif

