// ****************************************************************************
// copyright (c) 2000,2001 Horst Knorr <hk_classes@knorrnet.de>
// This file is part of the hk_kdeclasses library.
// This file may be distributed and/or modified under the terms of the
// GNU General Public License version 2 as published by the Free Software
// Foundation and appearing in the file COPYING included in the
// packaging of this file.
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
// ****************************************************************************
// $Revision: 1.7 $

#include "hk_kdegridcolumndialog.h"
#include <qlistbox.h>
#include <qlistview.h>
#include <qtoolbutton.h>
#include <qlabel.h>
#include <hk_kdegrid.h>
#include <hk_class.h>
/*
 *  Constructs a hk_kdegridcolumndialog which is a child of 'parent', with the
 *  name 'name' and widget flags set to 'f'
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
 */
hk_kdegridcolumndialog::hk_kdegridcolumndialog( hk_kdegrid* parent,  const char* name, bool modal, WFlags fl )
    : hk_kdegridcolumndialogbase( parent, name, modal, fl )
{
   p_grid=parent;
   set_columnvalues();
   leftlabel->setText(hk_class::hk_translate("Invisible columns:").c_str());
   rightlabel->setText(hk_class::hk_translate("Shown columns:").c_str());
    rightbox->addColumn( hk_class::hk_translate( "Column" ).c_str() );
    rightbox->addColumn( hk_class::hk_translate( "Default value" ).c_str() );
   setCaption(hk_class::hk_translate("Column select").c_str());

}

/*
 *  Destroys the object and frees any allocated resources
 */
hk_kdegridcolumndialog::~hk_kdegridcolumndialog()
{
    // no need to delete child widgets, Qt does it all for us
}

/*
 * public slot
 */
void hk_kdegridcolumndialog::addbutton_clicked()
{
    if (leftbox->currentText().isEmpty())return;
//    rightbox->insertItem(leftbox->currentText(),rightbox->currentItem()+1);
    QListViewItem* feld=new QListViewItem(rightbox,leftbox->currentText());
    leftbox->removeItem(leftbox->currentItem());
    listentries_changed();
}
/*
 * public slot
 */
void hk_kdegridcolumndialog::deletebutton_clicked()
{
    if (rightbox->currentItem()==NULL)return;
    leftbox->insertItem(rightbox->currentItem()->text(0),leftbox->currentItem()+1);
//    rightbox->removeItem(rightbox->currentItem());
    delete rightbox->currentItem();
    listentries_changed();
}
/*
 * public slot
 */
void hk_kdegridcolumndialog::exitbutton_clicked()
{
 close();
}
/*
 * public slot
 */
void hk_kdegridcolumndialog::okbutton_clicked()
{
    accept();
}


void hk_kdegridcolumndialog::listentries_changed(void)
{
    addbutton->setEnabled(leftbox->count()>0);
    deletebutton->setEnabled(rightbox->childCount()>0);
    storebutton->setEnabled(rightbox->childCount()>0);
}

void hk_kdegridcolumndialog::set_columnvalues()
{
   if (p_grid==NULL) return;

   leftbox->clear();
   rightbox->clear();
   list<hk_string>* p_visiblelist =p_grid->visible_columns();
   list<hk_string>::iterator it=p_visiblelist->begin();
   int i=0;
   QString def;
   while (it!=p_visiblelist->end())
   {
       hk_dsgridcolumn* gc=p_grid->gridcolumn(i);
       if (gc==NULL) def="";
       else def=gc->raw_defaultvalue().c_str();
       QListViewItem* feld=new QListViewItem(rightbox,(*it).c_str(),def);
     // rightbox->insertItem((*it).c_str());
      it++;i++;
   }

   list<hk_string>* p_nonvisiblelist =p_grid->nonvisible_columns();
   it=p_nonvisiblelist->begin();

   while (it!=p_nonvisiblelist->end())
   {
      leftbox->insertItem((*it).c_str());
      it++;
   }


   listentries_changed();
}

void hk_kdegridcolumndialog::defaultvalue_changed()
{
  rightbox->currentItem()->setText(1,defaultfield->text());

}

void hk_kdegridcolumndialog::rightbox_selectionchanged(QListViewItem* i)
{
   defaultfield->blockSignals(true);
   defaultfield->setText( rightbox->currentItem()->text(1));
   defaultfield->blockSignals(false);
}

