// ****************************************************************************
// copyright (c) 2000,2001 Horst Knorr <hk_classes@knorrnet.de>
// This file is part of the hk_kdeclasses library.
// This file may be distributed and/or modified under the terms of the
// GNU General Public License version 2 as published by the Free Software
// Foundation and appearing in the file COPYING included in the
// packaging of this file.
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
// ****************************************************************************
//$Revision: 1.32 $
#include "hk_kdeproperty.h"
#include "hk_visible.h"
#include "hk_dsvisible.h"
#include "hk_dsdatavisible.h"
#include "hk_kdeformdatasourcedialog.h"
#include "hk_kdesimpleform.h"
#include <hk_database.h>
#include <hk_button.h>
#include <qcombobox.h>
#include <qspinbox.h>
#include <qfontdatabase.h>
#include <qlineedit.h>
#include <qlabel.h>
#include <qtabwidget.h>
#include <qtoolbutton.h>
#include <hk_datasource.h>
#include <hk_dscombobox.h>
/*
 *  Constructs a hk_kdeproperty which is a child of 'parent', with the
 *  name 'name' and widget flags set to 'f'
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
 */
hk_kdeproperty::hk_kdeproperty( hk_kdesimpleform* parent,  const char* name, bool modal, WFlags fl )
    : hk_kdepropertyeditorbase( parent, name, modal, fl )  , hk_class()
{
#ifdef HK_DEBUG
hkdebug("hk_kdeproperty::hk_kdeproperty");
#endif
   p_form=parent;
   p_visible=NULL;
   widthfield->setRange(0,10000);
   heightfield->setRange(0,10000);
   xfield->setRange(0,10000);
   yfield->setRange(0,10000);
   p_fontdatabase = new QFontDatabase();
   fontfield->insertStringList(p_fontdatabase->families());
   actionfield->insertItem(hk_translate("Open form").c_str());
   actionfield->insertItem(hk_translate("Close form").c_str());
   actionfield->insertItem(hk_translate("Open table").c_str());
   actionfield->insertItem(hk_translate("Open query").c_str());
   actionfield->insertItem(hk_translate("Preview report").c_str());
   actionfield->insertItem(hk_translate("Print report").c_str());
   actionfield->insertItem(hk_translate("Goto first row").c_str());
   actionfield->insertItem(hk_translate("Goto last row").c_str());
   actionfield->insertItem(hk_translate("Goto next row").c_str());
   actionfield->insertItem(hk_translate("Goto previous row").c_str());
   actionfield->insertItem(hk_translate("Insert row").c_str());
   actionfield->insertItem(hk_translate("Delete row").c_str());
   actionfield->insertItem(hk_translate("Store row").c_str());


   actionfield->setCurrentItem(0);
   tab->changeTab(tab_2,hk_translate("&Format").c_str());
   tab->changeTab(tab_3,hk_translate("&Data").c_str());
   tab->changeTab(tab_4,hk_translate("&Action").c_str());
   labellabel->setText(hk_translate("Label:").c_str());
   xlabel->setText(hk_translate("X:").c_str());
   ylabel->setText(hk_translate("Y:").c_str());
   widthlabel->setText(hk_translate("Width:").c_str());
   heightlabel->setText(hk_translate("Height:").c_str());
   fontlabel->setText(hk_translate("Font:").c_str());
   fontsizelabel->setText(hk_translate("Fontsize:").c_str());
   datasourcelabel->setText(hk_translate("Datasource:").c_str());
   columnlabel->setText(hk_translate("Column:").c_str());
   actionlabel->setText(hk_translate("Action:").c_str());
   objectlabel->setText(hk_translate("Object:").c_str());
   defaultlabel->setText(hk_translate("Default value:").c_str());

   listdatasourcelabel->setText(hk_translate("Listdatasource:").c_str());
   listcolumnlabel->setText(hk_translate("Listcolumn:").c_str());
   viewcolumnlabel->setText(hk_translate("Viewcolumn:").c_str());

   showmaximizedlabel->setText(hk_translate("show maximized:").c_str());
   showmaximizedfield->insertItem(hk_translate("no").c_str());
   showmaximizedfield->insertItem(hk_translate("yes").c_str());

}

/*
 *  Destroys the object and frees any allocated resources
 */
hk_kdeproperty::~hk_kdeproperty()
{
#ifdef HK_DEBUG
hkdebug("hk_kdeproperty::~hk_kdeproperty");
#endif
    // no need to delete child widgets, Qt does it all for us
    delete  p_fontdatabase;
}

/*
 * public slot
 */
void hk_kdeproperty::action_changes()
{
#ifdef HK_DEBUG
hkdebug("hk_kdeproperty::action_changes()");
#endif
    if (p_form==NULL) return;
    if (p_form->database()==NULL)return;
    set_actionobjectlist();
    set_objectaction();
}


void hk_kdeproperty::set_actionobjectlist(void)
{
    objectfield->clear();
    int i = actionfield->currentItem();
    vector<hk_string>* liste=NULL;
    if (i==2)  liste=p_form->database()->tablelist();
    else
    if (i==3)  liste=p_form->database()->querylist();
    else
    if (i==4||i==5)  liste=p_form->database()->reportlist();
    else
    if (i==0||i==1)  liste=p_form->database()->formlist();
    if (liste==NULL) return;
    vector<hk_string>::iterator it =  liste->begin();
    while (it!=liste->end())
    {
     objectfield->insertItem((*it).c_str());
     it++;
    }
    objectfield->setCurrentItem(0);



}

void hk_kdeproperty::actionobject_changes()
{
#ifdef HK_DEBUG
hkdebug("hk_kdeproperty::action_changes()");
#endif
if (p_visible==NULL) return;
    if (p_visible->type()==hk_visible::button) set_objectaction();
}
/*
 * public slot
 */
void hk_kdeproperty::data_changes()
{
#ifdef HK_DEBUG
hkdebug("hk_kdeproperty::data_changes()");
#endif
   if (p_visible==NULL) return;
   switch (p_visible->type())
   {
      case hk_visible::lineedit:;
      case hk_visible::boolean:;
      case hk_visible::memo:;
      case hk_visible::combobox:;
      				       set_objectdsdatavisible();

      case hk_visible::form:
      case hk_visible::grid:
      case hk_visible::button:
      			            set_objectdsvisible();
      default:;

   }
}
/*
 * public slot
 */
void hk_kdeproperty::format_changes()
{
#ifdef HK_DEBUG
hkdebug("hk_kdeproperty::format_changes()");
#endif
    set_objectvisible();
}
/*
 * public slot
 */
void hk_kdeproperty::set_object(hk_visible* v)
{
#ifdef HK_DEBUG
hkdebug("hk_kdeproperty::set_object");
#endif
   p_visible=v;
   tab_2->setEnabled(false);
   tab_3->setEnabled(false);
   tab_4->setEnabled(false);
   xfield->setEnabled(true);
   yfield->setEnabled(true);
   fontfield->setEnabled(true);
   fontsizefield->setEnabled(true);
   if (p_visible==NULL) return;
   QString head;
   head.setNum(p_visible->presentationnumber());

   setCaption(hk_translate("Property editor (").c_str()+head+")");
   tab_2->setEnabled(true);
   columnfield->setEnabled(false);
   defaultfield->setEnabled(false);
   listcolumnfield->setEnabled(false);
   viewcolumnfield->setEnabled(false);
   listdatasourcefield->setEnabled(false);
   listdatasourcebutton->setEnabled(false);
   widthlabel->setText(hk_translate("Width:").c_str());
   heightlabel->setText(hk_translate("Height:").c_str());
   set_visible();
   switch (p_visible->type())
   {
      case hk_visible::grid:            set_dsvisible();
      					tab_3->setEnabled(true);
					set_gridvisible();
					break;
      case hk_visible::combobox:	listcolumnfield->setEnabled(true);
      					viewcolumnfield->setEnabled(true);
					listdatasourcefield->setEnabled(true);
					listdatasourcebutton->setEnabled(true);
      case hk_visible::lineedit:;
      case hk_visible::boolean:;
      case hk_visible::memo:;
					defaultfield->setEnabled(true);
					set_dsvisible();
					set_dsdatavisible();
					columnfield->setEnabled(true);
					tab_3->setEnabled(true);
      					break;

      case  hk_visible::form:
      					set_dsvisible();
					widthlabel->setText(hk_translate("Form width:").c_str());
					heightlabel->setText(hk_translate("Form height:").c_str());
					xfield->setEnabled(false);
   					fontfield->setEnabled(false);
   					fontsizefield->setEnabled(false);
					yfield->setEnabled(false);
					tab_3->setEnabled(true);
      					break;
      case  hk_visible::button:		set_button();
      					set_dsvisible();
      					tab_3->setEnabled(true);
					tab_4->setEnabled(true);
      					break;
      case  hk_visible::textlabel:	tab_3->setEnabled(false);
      					break;
      default:;

   }


}

void hk_kdeproperty::focus_resized(void)
{
#ifdef HK_DEBUG
hkdebug("hk_kdeproperty::focus_resized");
#endif
    set_visible();
}

void hk_kdeproperty::set_visible(void)
{
#ifdef HK_DEBUG
hkdebug("hk_kdeproperty::set_visible");
#endif
   if (p_visible==NULL)return;
   int w=0;
   int h=0;
   if (p_visible->type()==hk_visible::form)
   {
     hk_form* f=dynamic_cast <hk_form*>(p_visible);
     if (f!=NULL)
     {
	w=f->designwidth();
	h=f->designheight();
     }
   }
   else
   {
     w=p_visible->width();
     h=p_visible->height();

   }
   widthfield->blockSignals(true);
   widthfield->setValue(w);
   widthfield->blockSignals(false);
   heightfield->blockSignals(true);
   heightfield->setValue(h);
   heightfield->blockSignals(false);
   xfield->blockSignals(true);
   xfield->setValue(p_visible->x());
   xfield->blockSignals(false);
   yfield->blockSignals(true);
   yfield->setValue(p_visible->y());
   yfield->blockSignals(false);
   fontsizefield->blockSignals(true);
   fontsizefield->setValue(p_visible->fontsizeint());
   fontsizefield->blockSignals(false);
   fontfield->blockSignals(true);
   fontfield->setEditText(p_visible->font().c_str());
   fontfield->blockSignals(false);
   labelfield->blockSignals(true);
   labelfield->setText(p_visible->label().c_str());
   labelfield->blockSignals(false);


}

void hk_kdeproperty::set_dsvisible(void)
{
#ifdef HK_DEBUG
hkdebug("hk_kdeproperty::set_dsvisible");
#endif
   if (p_visible==NULL)return;
   set_datasourcelist();
     columnfield->blockSignals(true);
     columnfield->clear();
     columnfield->blockSignals(false);
     datasource_selected();
     listdatasource_selected();
}

void hk_kdeproperty::set_dsdatavisible(void)
{
#ifdef HK_DEBUG
hkdebug("hk_kdeproperty::set_dsdatavisible");
#endif
   if (p_visible==NULL)return;
   hk_dsdatavisible* ds= dynamic_cast <hk_dsdatavisible*>(p_visible);
   if (ds==NULL) {return;}
   columnfield->blockSignals(true);
   columnfield->setEditText(ds->columnname().c_str());
   columnfield->blockSignals(false);
   defaultfield->blockSignals(true);
   defaultfield->setText(ds->raw_defaultvalue().c_str());
   defaultfield->blockSignals(false);
   hk_dscombobox* combo=dynamic_cast <hk_dscombobox*>(p_visible);
   if (combo==NULL) return;
   listcolumnfield->blockSignals(true);
   listcolumnfield->setEditText(combo->listcolumnname().c_str());
   listcolumnfield->blockSignals(false);
   viewcolumnfield->blockSignals(true);
   viewcolumnfield->setEditText(combo->viewcolumnname().c_str());
   viewcolumnfield->blockSignals(false);



}


void hk_kdeproperty::set_gridvisible(void)
{
#ifdef HK_DEBUG
hkdebug("hk_kdeproperty::set_gridvisible");
#endif
   if (p_visible==NULL)return;
}


void hk_kdeproperty::set_button(void)
{
#ifdef HK_DEBUG
hkdebug("hk_kdeproperty::set_button");
#endif
   if (p_visible==NULL)return;
    hk_button* bt = dynamic_cast <hk_button*>(p_visible);
   if (bt==NULL) return;

    int i = bt->action();
    showmaximizedfield->blockSignals(true);
    showmaximizedfield->setCurrentItem(bt->show_maximized()?1:0);
    showmaximizedfield->blockSignals(false);
    actionfield->setCurrentItem(i);
    set_actionobjectlist();
    vector<hk_string>* liste=NULL;
    if (i==3)  liste=p_form->database()->querylist();
    else
    if (i==4||i==5)  liste=p_form->database()->reportlist();
    else
   if (i==0||i==1)  liste=p_form->database()->formlist();
    else
      liste=p_form->database()->tablelist();
    vector<hk_string>::iterator it =  liste->begin();
    int current=0;
    while (it!=liste->end())
    {

     if ((*it)==bt->object())
       {
          objectfield->setCurrentItem(current);
       }
     it++;  current++;
    }


}

void hk_kdeproperty::set_objectvisible(void)
{
#ifdef HK_DEBUG
hkdebug("hk_kdeproperty::set_objectvisible");
#endif
   if (p_visible==NULL)return;
   if (p_visible->type()==hk_visible::form)
   {
     hk_form* f=dynamic_cast <hk_form*>(p_visible);
     if (f!=NULL)
     {
	f->set_formdesignsize(widthfield->value(),heightfield->value());
     }
   }
   else
   {
   p_visible->set_width(widthfield->value());
   p_visible->set_height(heightfield->value());

   }

   p_visible->set_x(xfield->value());
   p_visible->set_y(yfield->value());
   p_visible->set_label(labelfield->text().latin1());
   const char* s =  fontfield->currentText().latin1();

   hk_string ps;
   if (s!=NULL) ps=s;
   p_visible->set_font(ps,fontsizefield->value());


}

void hk_kdeproperty::set_objectdsvisible(void)
{
#ifdef HK_DEBUG
hkdebug("hk_kdeproperty::set_objectdsvisible");
#endif
   if (p_visible==NULL)return;

  list<hk_datasource*>* dslist = p_form->datasources();
  if (dslist==NULL) return;
  list<hk_datasource*>::iterator it=dslist->begin();
  hk_dsvisible* ds = dynamic_cast <hk_dsvisible*>(p_visible);
  hk_dscombobox* combo =dynamic_cast <hk_dscombobox*>(p_visible);
  if (ds==NULL) return;
  if (datasourcefield->currentText()=="") ds->set_presentationdatasource(-1);
  if (combo!=NULL&&listdatasourcefield->currentText()=="") combo->set_listpresentationdatasource(-1);
  while (it!=dslist->end())
  {
     QString name= p_form->unique_datasourcename((*it)->presentationnumber()).c_str();
     if (name==datasourcefield->currentText())
       {
         ds->set_presentationdatasource( (*it)->presentationnumber() );
       }
   if (combo!=NULL)
   {
     if (name==listdatasourcefield->currentText())
       {
         combo->set_listpresentationdatasource( (*it)->presentationnumber() );
       }



   }

     it++;
  }
}

void hk_kdeproperty::set_objectdsdatavisible(void)
{
#ifdef HK_DEBUG
hkdebug("hk_kdeproperty::set_objectdsdatavisible");
#endif
   if (p_visible==NULL)return;
   hk_dsdatavisible* ds= dynamic_cast <hk_dsdatavisible*>(p_visible);
   ds->set_columnname(columnfield->currentText().latin1());
   if (defaultfield->text().isEmpty()) ds->reset_default();
   else ds->set_defaultvalue(defaultfield->text().latin1());
  hk_dscombobox* combo =dynamic_cast <hk_dscombobox*>(p_visible);
  if (combo==NULL) return;
  combo->set_viewcolumnname(viewcolumnfield->currentText().latin1());
  combo->set_listcolumnname(listcolumnfield->currentText().latin1());
}

void hk_kdeproperty::set_objectaction(void)
{
#ifdef HK_DEBUG
hkdebug("hk_kdeproperty::set_objectaction");
#endif
   if (p_visible==NULL)return;
  hk_button* bt = dynamic_cast <hk_button*>(p_visible);
  if (bt==NULL) return;
   bt->set_action(
   		  actionfield->currentItem(),
   		  objectfield->currentText().isEmpty()?"":objectfield->currentText().latin1(),
		  (showmaximizedfield->currentItem()==1?true:false )
		 );
}

void hk_kdeproperty::add_formdatasource()
{
#ifdef HK_DEBUG
hkdebug("hk_kdeproperty::add_formdatasource");
#endif
     hk_kdeformdatasourcedialog* d = new hk_kdeformdatasourcedialog(p_form,this,0,true);
     d->set_datasource(p_form->get_datasource(datasourcefield->currentText().latin1()));
     d->exec();
     delete d;
     set_datasourcelist();
}

void hk_kdeproperty::add_listformdatasource()
{
#ifdef HK_DEBUG
hkdebug("hk_kdeproperty::add_listformdatasource");
#endif
     hk_kdeformdatasourcedialog* d = new hk_kdeformdatasourcedialog(p_form,this,0,true);
     d->set_datasource(p_form->get_datasource(listdatasourcefield->currentText().latin1()));
     d->exec();
     delete d;
     set_datasourcelist();
}

void hk_kdeproperty::set_datasourcelist(void)
{
#ifdef HK_DEBUG
hkdebug("hk_kdeproperty::set_datasourcelist");
#endif
  if (p_form==NULL) return;
  datasourcefield->blockSignals(true);
  listdatasourcefield->blockSignals(true);
  datasourcefield->clear();
  datasourcefield->insertItem("");
  listdatasourcefield->clear();
  listdatasourcefield->insertItem("");

  list<hk_datasource*>* dslist = p_form->datasources();
  if (dslist==NULL)
     {
	 datasourcefield->blockSignals(false);
	 listdatasourcefield->blockSignals(false);
	 return;
     }
  list<hk_datasource*>::iterator it=dslist->begin();
  int item=1;
  while (it!=dslist->end())
  {
     QString name= p_form->unique_datasourcename((*it)->presentationnumber()).c_str();
     datasourcefield->insertItem(name);
     listdatasourcefield->insertItem(name);
     hk_dsvisible* ds = dynamic_cast <hk_dsvisible*>(p_visible);
     hk_dscombobox* dscombo = dynamic_cast <hk_dscombobox*>(p_visible);
     if (ds!=NULL)
     {
         if (ds->datasource()==(*it)) datasourcefield->setCurrentItem(item);
         if (dscombo!=NULL)if (dscombo->listdatasource()==(*it))
	 			 listdatasourcefield->setCurrentItem(item);

     }

     it++; item++;
  }
  datasourcefield->blockSignals(false);
  listdatasourcefield->blockSignals(false);
}

void hk_kdeproperty::datasource_selected(void)
{
#ifdef HK_DEBUG
hkdebug("datasource_selected");
#endif
   columnfield->blockSignals(true);
   columnfield->clear();
   columnfield->blockSignals(false);
   if (p_form==NULL) return;
   hk_database* db=p_form->database();
   if (db==NULL) return;
   QString f=datasourcefield->currentText();
   if (f.isEmpty()) f="";
   hk_datasource* ds= p_form->get_datasource(f.latin1());
   if (ds==NULL) return;
   list<hk_column*>* v=NULL;
   v=ds->columns();
   if (v==NULL) return;
   columnfield->insertItem("");
   list<hk_column*>::iterator it=v->begin();
   while (it!=v->end())
   {
    columnfield->insertItem((*it)->name().c_str());
    it++;
   }

}

void hk_kdeproperty::listdatasource_selected(void)
{
#ifdef HK_DEBUG
hkdebug("hk_kdeproperty::listdatasource_selected");
#endif
   listcolumnfield->blockSignals(true);
   listcolumnfield->clear();
   listcolumnfield->blockSignals(false);
   viewcolumnfield->blockSignals(true);
   viewcolumnfield->clear();
   viewcolumnfield->blockSignals(false);
   if (p_form==NULL) return;
   hk_database* db=p_form->database();
   if (db==NULL) return;
   QString f=listdatasourcefield->currentText();
   if (f.isEmpty()) f="";
   hk_datasource* ds= p_form->get_datasource(f.latin1());
   if (ds==NULL) return;
   list<hk_column*>* v=NULL;
   v=ds->columns();
   if (v==NULL) return;
   listcolumnfield->insertItem("");
   viewcolumnfield->insertItem("");
   list<hk_column*>::iterator it=v->begin();
   while (it!=v->end())
   {
    listcolumnfield->insertItem((*it)->name().c_str());
    viewcolumnfield->insertItem((*it)->name().c_str());
    it++;
   }



}


