// ****************************************************************************
// copyright (c) 2000,2001 Horst Knorr <hk_classes@knorrnet.de>
// This file is part of the hk_kdeclasses library.
// This file may be distributed and/or modified under the terms of the
// GNU General Public License version 2 as published by the Free Software
// Foundation and appearing in the file COPYING included in the
// packaging of this file.
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
// ****************************************************************************
//$Revision: 1.14 $

#include "hk_kdequery.h"
#include "hk_kdequery.moc"
#include "hk_kdetoolbar.h"
#include <qbuttongroup.h>
#include <qpushbutton.h>
#include <qtable.h>
#include <qtoolbutton.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qimage.h>
#include <qpixmap.h>
#include <qmultilineedit.h>
#include <klocale.h>
#include <hk_actionquery.h>


/*
 *  Constructs a hk_kdequery which is a child of 'parent', with the
 *  name 'name' and widget flags set to 'f'
 */

hk_kdequery::hk_kdequery(QWidget* w,const char* n,WFlags f):QWidget(w,n,f)
{
    p_sql_has_changed =false;
    setName( "hk_kdequery" );
    resize( 596, 480 );
    //setCaption( tr( ""  ) );
    hk_kdequeryLayout = new QVBoxLayout( this );
    hk_kdequeryLayout->setSpacing( 0 );
    hk_kdequeryLayout->setMargin( 0 );

    toolbar = new hk_kdetoolbar( this, "toolbar" );
    //toolbar->setTitle( tr( ""  ) );


    hk_kdequeryLayout->addWidget( toolbar );
    stack= new QWidgetStack(this);
    stack->setSizePolicy(QSizePolicy(QSizePolicy::Expanding,QSizePolicy::Expanding));
    hk_kdequeryLayout->addWidget( stack );
    p_grid = new hk_kdegrid(stack);
    p_grid->set_enablingbehaviour(true,true);
    p_design = new QMultiLineEdit(stack);
    p_qbe = new QWidget(stack);
    stack->addWidget(p_design,1);
    stack->addWidget(p_grid,2);
    stack->addWidget(p_qbe,3);
    connect( toolbar, SIGNAL( signal_designmode_selected() ), this, SLOT( designbutton_clicked() ) );
    connect( toolbar, SIGNAL( signal_viewmode_selected() ), this, SLOT( querybutton_clicked() ) );
    connect( toolbar, SIGNAL( signal_savebutton_clicked() ), this, SLOT( savebutton_clicked() ) );
    connect( p_design, SIGNAL( textChanged() ), this, SLOT( query_changed() ) );
    connect( toolbar, SIGNAL( signal_searchbutton_clicked() ), p_grid, SLOT( find_clicked() ) );
    p_autoclose =true;
    designbutton_clicked();
    setCaption(hk_translate("Querywindow").c_str());
    set_nodesignmode(runtime_only());
}

/*
 *  Destroys the object and frees any allocated resources
 */
hk_kdequery::~hk_kdequery()
{
    // no need to delete child widgets, Qt does it all for us
 hk_datasource* d=p_grid->datasource();
 if (d!=NULL)
 {
   d->disable();
//   hk_string tp_design->text();
 }
}

void hk_kdequery::set_datasource(hk_datasource* d)
{
  hk_dsvisible::set_datasource(d);
  p_grid->set_datasource(d);
  p_design->setText(d->sql().c_str());
  toolbar->set_datasource(d);
  if (d!=NULL)
  {
   d->enable();
//   setCaption(d->name());
  }
 p_sql_has_changed=false;
}

hk_datasource* hk_kdequery::datasource(void)
{

  return hk_dsvisible::datasource();
}

void hk_kdequery::designbutton_clicked(void)
{
   if (p_nodesignmode)
   {
      querybutton_clicked();
      return;
   }
   hk_datasource* d=p_grid->datasource();
   if (d!=NULL) d->disable();
   stack->raiseWidget(1);
   p_designmode=true;
   toolbar->set_designmode();

}

void hk_kdequery::querybutton_clicked(void)
{
   hk_datasource* d=p_grid->datasource();
   hk_string tst_sql=p_design->text().latin1();
   for (unsigned int tt=0;tt<tst_sql.size();tt++)
 	{
 	 tst_sql[tt]=toupper(tst_sql[tt]);
 	}
    int pos=tst_sql.find("ALTER");
    if (pos<0) pos=tst_sql.find("DROP");
    if (pos<0) pos=tst_sql.find("INSERT");
    if (pos<0) pos=tst_sql.find("UPDATE");
    if (pos<0) pos=tst_sql.find("CREATE");

   if (d!=NULL)
   {
    if (pos<0)
      {d->set_sql(p_design->text().latin1(),true);
       d->enable();
      }
      else
      {
       hk_actionquery* q=d->database()->new_actionquery();
       if (q!=NULL)
       {
          hk_string sql=p_design->text().latin1();
	 q->set_sql(sql.c_str(),sql.length());
	 if (q->execute())
	   show_warningmessage(hk_translate("Query was successful"));
	 else
	   show_warningmessage(hk_translate("Query could not be executed"));
       delete q;
       if (!runtime_only())
         {
          set_sqlmode();
          return;
	 }
       }

      }
   }
   stack->raiseWidget(2);
   p_designmode=false;
   toolbar->set_viewmode();

}


void hk_kdequery::set_sqlmode(void)
{
   designbutton_clicked();
}

void hk_kdequery::set_viewmode(void)
{
   querybutton_clicked();
}

void hk_kdequery::set_designmode(void)
{
   set_sqlmode();

}



void hk_kdequery::before_source_vanishes(void)
{
#ifdef HK_DEBUG
 hkdebug("hk_kdequery::before_source_vanishes");
#endif

  if (p_autoclose)
    close();
  else
  {
   hk_datasource* d=datasource();
   if (d!=NULL)d->disable();
  p_grid->set_datasource(NULL);
  }
}

void hk_kdequery::query_changed(void)
{
 p_sql_has_changed=true;
  if (datasource()!=NULL)  datasource()->set_sql(p_design->text().latin1(),true);
}


void hk_kdequery::closeEvent ( QCloseEvent* e)
{
   if (p_sql_has_changed)
    {
       if (p_grid->datasource()!=NULL)  p_grid->save_query();
       p_sql_has_changed=false;
    }
 QWidget::closeEvent(e);

}

void hk_kdequery::set_nodesignmode(bool d)
{
 toolbar->set_nodesignmode(d);
 p_nodesignmode=d;

}

bool hk_kdequery::save_query(const hk_string& n)
{
  if (toolbar->is_filter_activated())
 	toolbar->deactivate_filter();

 return p_grid->save_query(n);
}

bool hk_kdequery::load_query(void)
{
 bool r= p_grid->load_query();
   hk_datasource* d=datasource();
   if (d==NULL)return false;
   p_design->blockSignals(true);
   p_design->setText(d->sql().c_str());
   p_design->blockSignals(false);
   return r;
}

void hk_kdequery::savebutton_clicked(void)
{
   if (p_grid->save_query("",false))   p_sql_has_changed=false;
}
