// ****************************************************************************
// copyright (c) 2000,2001 Horst Knorr <hk_classes@knorrnet.de>
// This file is part of the hk_kdeclasses library.
// This file may be distributed and/or modified under the terms of the
// GNU General Public License version 2 as published by the Free Software
// Foundation and appearing in the file COPYING included in the
// packaging of this file.
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
// ****************************************************************************
//$Revision: 1.21 $

#include <hk_kdesimplegrid.h>
#include <hk_kdesimplegrid.moc>
#include <qdrawutil.h>
#include <list>
#include <qscrollbar.h>
#include <qkeycode.h>
#include <qpainter.h>
#include <qfontmetrics.h>
#include <qapplication.h>
#include <qclipboard.h>
#include <qapplication.h>
#include <qpopupmenu.h>
#include <qlistbox.h>
#include <hk_kdesimpleform.h>
#include <hk_kdegrid.h>
#include <hk_kdegridcolumndialog.h>
#include "hk_dsgridcolumn.h"
#include <qlabel.h>
#include <qlistview.h>
static const int MINSIZE  = 12;

/*
*Xpostocursopos and showlastpartoffset were taken from QT QLineedit
* all QString tbuf's are replaced by p_actualcelltext. Just an important note if >=QT3.0 is again not compatible
*/

int hk_kdesimplegrid::xPosToCursorPos( int goalx )
{
#ifdef HK_DEBUG
hkdebug("hk_kdesimplegrid::xpostocursorpos");
#endif
    int x1, x2;
    x1 = offset;
    int i = 0;
    QFontMetrics fm = fontMetrics();
    QString s = p_actualcelltext;
    goalx -=2;

    while( i < (int) s.length() ) {
	x2 = x1 + fm.width( s[i] );
	if ( QABS( x1 - goalx ) < QABS( x2 - goalx ) )
	    return i;
	i++;
	x1 = x2;
    }
    return i;
}

int hk_kdesimplegrid::showLastPartOffset(char*s,const QFontMetrics &fm,int width)
{
#ifdef HK_DEBUG
hkdebug("hk_kdesimplegrid::showLastPartOffset");
#endif
  if (!s||s[0]=='\0') return 0;
  char* tmp=&s[strlen(s)-1];
  do
  {
   width-=fm.width(tmp--,1);

  }   while (tmp>=s&&width>=0);
  return width<0?tmp-s+2:0;

}

//*******************************
//***       hk_header         ***
//*******************************
hk_header::hk_header(QWidget* w):QHeader(w)
{
#ifdef HK_DEBUG
//cout <<"hk_header::hk_header"<<endl;
#endif
	p_actualcell=0;
	p_rows=1;
	p_height=default_cellheight;
	p_grid=0;
	p_asterics=false;
}
void hk_header::set_rownumbers(unsigned long int n)
{
#ifdef HK_DEBUG
//cout <<"hk_header::set_rownumbers"<<endl;
#endif
	if ((unsigned )count()<n)
        {
         unsigned long int dif=n-count();
         for (unsigned long int k=0;k<dif;k++) addLabel("",p_height);
        }

	if (p_rows<n)
	    for (unsigned long int k=p_rows; k<n;k++) setLabel(k,"",p_height);
	if (p_rows>n)
//	    for (unsigned long int k=n; k<p_rows;k++) setLabel(k,"",0);
            {
	      while ((unsigned)count()>n)
	      {
		 if (count()>0) removeLabel(count()-1);

	      }

	    }
	p_rows=n;

}

void hk_header::set_asterics(bool a)
{
#ifdef HK_DEBUG
//cout <<"hk_header::set_asterics"<<endl;
#endif
 p_asterics=a;
}

void hk_header::set_grid(hk_kdesimplegrid* g)
{
#ifdef HK_DEBUG
//cout <<"hk_header::set_grid"<<endl;
#endif
 p_grid=g;
}
//*******************************
//***       hk_vheader        ***
//*******************************


hk_vheader::hk_vheader(QWidget* w):QTableView(w)
{
  init(0);
  p_grid=NULL;
  p_actualcell=0;
  p_asterics=false;
  p_tracking=false;
  p_moving=false;
  p_height=default_cellheight;
  p_splitcursor=false;
  p_mousekeypressed=false;
  p_resizestartposition=0;
  p_resizerow=0;
  setTableFlags(Tbl_scrollLastVCell);
}

void hk_vheader::set_tracking(bool enable)
{
 p_tracking=enable;

}
void hk_vheader::set_grid(hk_kdesimplegrid* g)
{
#ifdef HK_DEBUG
//cout <<"hk_vheader::set_grid"<<endl;
#endif
 p_grid=g;
}

void hk_vheader::set_movingenabled(bool m)
{

 p_moving=m;
}

void hk_vheader::set_offset(int i)
{

//scroll(0,yOffset()-i);
setYOffset(i);
}


void hk_vheader::paintCell(class QPainter* painter,int y,int )
{
style().drawBevelButton( painter,0,0,width(),cellHeight(),colorGroup(), false );
	if((unsigned int)y==p_actualcell)
	       {
                qDrawArrow(painter,RightArrow,style(),true,3,3,width()-8,cellHeight(y)-8,colorGroup(),isEnabled());
	       }
	else
	   if (p_grid!=0&&p_asterics)
 	   { if (p_grid->datasource()!=0)
 	      if ((unsigned int)y==p_grid->datasource()->max_rows())
 	      {
	        qDrawShadePanel(painter,0,0,width(),cellHeight(),colorGroup(),false,1);
				int x,y,w,h;
				x=2;y=2;w=width();h=cellHeight();
				int Lx =w / 2 -3;
				int Ly= h/2 ;
				int l=5;
				QPen newpen =painter->pen();
				newpen.setWidth(2);
				painter->setPen(newpen);
				painter->setPen(colorGroup().foreground());
				painter->drawLine(Lx,Ly,Lx+l+1,Ly);
				painter->drawLine(w/2,Ly-3,w/2,Ly+3);

 	      }
	 }
}

void hk_vheader::init(int rows)
{
        setSizePolicy( QSizePolicy( QSizePolicy::Fixed, QSizePolicy::Preferred ) );
        setFrameStyle( QFrame::NoFrame );
	setCellWidth( default_cellwidth );
	setCellHeight( default_cellheight );
	setNumCols( 1 );
	setNumRows( rows );
	setMouseTracking(true);
}

void hk_vheader::set_rownumbers(unsigned long int i)
{
//	set_cellheight(p_height);
	setNumRows( i );
	//if (i<2)
	set_offset(0);
}
void hk_vheader::set_cellwidth(int w)
{
setCellWidth(w);
}
void hk_vheader::set_asterics(bool a)
{
#ifdef HK_DEBUG
//cout <<"hk_vheader::set_asterics"<<endl;
#endif
 p_asterics=a;
}
void hk_vheader::set_cellheight(int h)
{
#ifdef HK_DEBUG
//cout <<"hk_vheader::set_cellheight"<<endl;
#endif
p_height=h;
setCellHeight(h);
}

void hk_vheader::mouseMoveEvent( QMouseEvent *e )
{
  //int section;
  bool hit=false;
  int y=e->pos().y();

	int i = 0;
	while ( i <= (int) numRows() )
	{
	    if ( i && ppos(i) - MINSIZE/2 < y && y < ppos(i) + MINSIZE/2 )
			hit = TRUE;
	   i++;
	}


  if (p_mousekeypressed)
  {
//   	    handle_resize( handleIdx, s, FALSE );
  int p_plannednewsize=e->pos().y()-p_resizestartposition+p_originalsize;
  p_newsize=p_plannednewsize>MINSIZE?p_plannednewsize:MINSIZE;
  setCellHeight(p_newsize);
  }
  else
   if (hit )
   {
     setCursor( splitVCursor );
     p_splitcursor=true;
   }
   else
   {
      unsetCursor();
      p_splitcursor=false;
   }


}
void hk_vheader::mousePressEvent( QMouseEvent *e )
{
   p_mousekeypressed=p_splitcursor;
   if (p_mousekeypressed)
   {
    p_resizestartposition=e->pos().y();
    p_resizerow=findRow(p_resizestartposition);
    p_originalsize=cellHeight();
    p_newsize=p_originalsize;

   }

}
void hk_vheader::mouseReleaseEvent( QMouseEvent * )
{
   if (p_mousekeypressed)
   {

      emit sizeChange( p_resizerow, p_originalsize, p_newsize );


   }
   p_mousekeypressed=false;
   unsetCursor();
}

/*void hk_vheader::handle_resize( int index, int s, bool final )
{
}
*/

/*!
  Returns the position of actual division line.  May return a position
  outside the widget.
 */
int hk_vheader::ppos( int i )
{
    int r = 0;
    bool ok;
	ok = rowYPos( i, &r );
    if ( !ok ) {
	r = 0;
	for ( int j = 0; j < i; j++ )
	    r += cellHeight();
	r -= yOffset();
    }
    return r;
}

int hk_vheader::totalHeight()
{
	return cellHeight()*numRows();

}

//*******************************
//***       hk_hheader        ***
//*******************************


 hk_hheader::hk_hheader(QWidget* w):hk_header(w)
 {
#ifdef HK_DEBUG
//cout <<"hk_hheader::hk_hheader"<<endl;
#endif
 setLabel(0,"",default_cellwidth);
 };



//*******************************
//***    hk_kdesimplegrid     ***
//*******************************





hk_kdesimplegrid::hk_kdesimplegrid(hk_kdegrid* grid, QWidget* wid,hk_kdesimpleform* form):QTableView(wid),
hk_dsvisible(form)
{

#ifdef HK_DEBUG
hkclassname("hk_kdesimplegrid");
hkdebug("hk_kdesimplegrid::hk_kdesimplegrid");
//set_generaldebug(true);
#endif
//	setBackgroundMode(PaletteMidlight);
        p_grid=grid;
	setBackgroundMode(PaletteDark);
	p_cellheight=default_cellheight;
 	setCellHeight(p_cellheight);
	setTableFlags(Tbl_clipCellPainting|Tbl_autoScrollBars);
	QScrollBar* scroll= (QScrollBar*)horizontalScrollBar();
	scroll->setTracking(true);
	scroll=(QScrollBar*) verticalScrollBar();
	scroll->setTracking(true);
	vheader=NULL;
	hheader=NULL;
	offset=0;
	cursorpos=0;
	p_actualcell_x=0;
	p_actualcell_y=0;
	p_cursoron=false;
	pmdirty=true;
	offsetdirty=true;
	p_datachanged=false;
	p_blinkspeed=500;
	p_cellscrollspeed=100;
	p_maxlen=32767;
	markdrag=0;
	markanchor=0;
	p_newcellenter=false;
	p_rowcolupdate=false;
	p_cellscrolling=false;
	p_scrollleft=false;
	p_margin=5;
	connect(&p_blinktimer, SIGNAL(timeout()),this,SLOT(blinkcursorslot()));
	connect(&p_cellscrolltimer, SIGNAL(timeout()),this,SLOT(cellscrollslot()));
	setFocusPolicy(QWidget::StrongFocus);
	p_actualcelltext=" ";
//	wanna_debug(true);
//  setTableFlags(Tbl_scrollLastVCell);
}

hk_kdesimplegrid::~hk_kdesimplegrid(void)
{
#ifdef HK_DEBUG
hkdebug("hk_kdesimplegrid::destructor");
#endif


}

void hk_kdesimplegrid::paintCell(class QPainter* painter,int y,int x)
{
#ifdef HK_DEBUG
//hkdebug("hk_kdesimplegrid::paintcell");
#endif
QColorGroup farbe=colorGroup();
	painter->fillRect(0,0,cellWidth(x),cellHeight(),farbe.light());
	qDrawShadePanel(painter,0,0,cellWidth(x),cellHeight(),farbe,false,1);

   if (datasource()!=NULL)
     if ((datasource()->is_enabled())&&((unsigned int)x<p_grid->columnscount()))
     {
      if (p_grid->gridcolumn(x)->columntype()==hk_dsgridcolumn::columnbool)
       paint_boolcell(painter,y,x);
      else
       paint_textcell (painter,y,x);

    }
}

void hk_kdesimplegrid::paint_textcell(class QPainter* painter,int y,int x)
{
#ifdef HK_DEBUG
//hkdebug("hk_kdesimplegrid::paint_textcell");
#endif
    update_offset();
    if (((unsigned)y==p_actualcell_y)&&((unsigned)x==p_actualcell_x))
{
 //actualcell paint cursor
     #ifdef HK_DEBUG
     //hkdebug("actualcell Begin");
     #endif
	 int margin=1;
	const QColorGroup & g = colorGroup();
	QBrush bg = g.brush((isEnabled()) ? QColorGroup::Base :
			    QColorGroup::Background);
	QFontMetrics fm = fontMetrics();
	int markBegin = minmark();
	int markEnd = maxmark();
        QRect cellrect=cellUpdateRect();
        int curytop=cellrect.y()+2;
        int curybot=cellrect.bottom()-2;
	int curxpos=2;
	QString display = p_actualcelltext;

	QString before = display.mid( 0, markBegin );
	QString marked = display.mid( markBegin, markEnd - markBegin );
	QString after = display.mid( markEnd, display.length() );

        int ypos=cellHeight()-margin -fm.descent()-1-(cellHeight()-2*margin-fm.height())/2;


	int xpos = offset + 2;
	int w;

	w = fm.width( before );
	if ( xpos < cellWidth(x) && xpos + w >= 0 ) {
	    painter->setPen( g.text() );
	    painter->drawText( xpos, ypos, before );
	}
	xpos += w;

	w = fm.width( marked );
	if ( xpos < cellWidth(x) && xpos + w >= 0 ) {
	    painter->fillRect( xpos, ypos-fm.ascent()-1, w, fm.height()+2,
			g.brush( QColorGroup::Highlight ) );
	    painter->setPen( g.highlightedText() );
	    painter->drawText( xpos, ypos, marked );
	}
	xpos += w;

	w = fm.width( after );
	if ( xpos < cellWidth(x) && xpos + w >= 0 ) {
	    painter->setPen( g.text() );
	    painter->drawText( xpos, ypos, after );
	}

	painter->setPen( g.text() );


//now paint the cursor
		     #ifdef HK_DEBUG
		     // hkdebug("actualcell vor Cursor");
		     #endif

		     if (p_cursoron)
		        {
		      #ifdef HK_DEBUG
		     // hkdebug("actualcell Cursormalen");
		     #endif

//			    QRect cursorrepaintrect=QRect(offset+2+fontMetrics.width(p_actualcelltext.left(cursorpos)),curytop,5,1);
			    curxpos=offset+2+fontMetrics().width(p_actualcelltext.left(cursorpos));
			    painter->drawLine(curxpos,curytop,curxpos,curybot);
		            if (style()!=WindowsStyle)
		            {
		               painter->drawLine(curxpos-2,curytop,curxpos+2,curytop);
		               painter->drawLine(curxpos-2,curybot,curxpos+2,curybot);
		            }

		        }
		      #ifdef HK_DEBUG
		    //  hkdebug("actualcell End");
		     #endif




}
else
     //not the actual cell => paint all other cells
     		    {
        		    if (p_grid->gridcolumn(x)->column()!=NULL)
			      {
        		       if ((unsigned)y==datasource()->max_rows())
				{
				  if (p_grid->gridcolumn(x)->use_defaultvalue()&& !p_grid->gridcolumn(x)->column()->has_changed())
				     painter->drawText(2,3,cellWidth(x)-p_margin,cellHeight(),AlignLeft,p_grid->gridcolumn(x)->defaultvalue().c_str());
				}
			       else
			       painter->drawText(2,3,cellWidth(x)-p_margin,cellHeight(),AlignLeft,p_grid->gridcolumn(x)->column()->asstring_at(y).c_str());

			      }
			    if (p_actualcell_y==datasource()->max_rows())
	                      {
					 if (p_grid->gridcolumn(x)->column()!=NULL)
					 if (p_grid->gridcolumn(x)->column()->has_changed())
  		 	                 {
					    painter->drawText(2,3,cellWidth(x)-p_margin,cellHeight(),AlignLeft,p_grid->gridcolumn(x)->column()->changed_data_asstring().c_str());
					 }
					 else
					 {
					   if (p_grid->gridcolumn(x)->use_defaultvalue()&&(unsigned int)y==p_actualcell_y)
					     painter->drawText(2,3,cellWidth(x)-p_margin,cellHeight(),AlignLeft,p_grid->gridcolumn(x)->defaultvalue().c_str());
					 }
	                      }
                     }




}


void hk_kdesimplegrid::paint_boolcell(class QPainter* painter,int y,int x)
{
#ifdef HK_DEBUG
hkdebug("hk_kdesimplegrid::paint_boolcell");
#endif
    QColorGroup farbe=colorGroup();
    int square=10;
    int left=(cellWidth(x)-square)/2;
    int top=(cellHeight()-square)/2 ;
//    painter->fillRect((cellWidth(x)-square)/2,(cellHeight()-square)/2,square,square,farbe.dark());
	if (x==(signed)p_actualcell_x&&y==(signed)p_actualcell_y&&(p_cursoron)||(x!=(signed)p_actualcell_x||y!=(signed)p_actualcell_y)||!hasFocus())
	 qDrawShadeRect(painter,left,top,square,square,farbe,false,1);

	if (p_grid->gridcolumn(x)!=NULL)
	 if (((unsigned)y==p_actualcell_y)&&((unsigned)x==p_actualcell_x) && (p_grid->gridcolumn(x)->column()->has_changed()))
	 {
	 if  (p_grid->gridcolumn(x)->column()->changed_data_asbool())
	{
          painter->drawLine(left+3,top+3,left+square-3,top+square-3);
          painter->drawLine(left+3,top+square-3,left+square-3,top+3);
        }

	 }
	 else
	 if  (p_grid->gridcolumn(x)->column()->asbool_at(y))
	{
          painter->drawLine(left+3,top+3,left+square-3,top+square-3);
          painter->drawLine(left+3,top+square-3,left+square-3,top+3);
        }
//painter->drawText(left+3,top-3,left+square,top+square,AlignLeft,"x");


}
const int hk_kdesimplegrid::minmark()
{
  #ifdef HK_DEBUG
  hkdebug ("minmark");
  #endif

   return markanchor < markdrag ?markanchor:markdrag;
}

const int hk_kdesimplegrid::maxmark()
{
  #ifdef HK_DEBUG
  hkdebug ("maxmark");
  #endif

   return markanchor > markdrag ?markanchor:markdrag;
}


int hk_kdesimplegrid::cellWidth(int col)
{
#ifdef HK_DEBUG
hkdebug("hk_kdesimplegrid::cellWidth column: ",col);
#endif
  	if (datasource()!=NULL)
  	if (((unsigned )col<p_grid->columnscount())&&(datasource()->is_enabled()))
  		return p_grid->gridcolumn(col)->columnwidth();
  return default_cellwidth;
}

void hk_kdesimplegrid::widget_specific_enable_disable()
{
#ifdef HK_DEBUG
hkdebug("kdesimplegrid::widget_specific_enable_disable");
#endif
 if (datasource()!=NULL)
 {
  #ifdef HK_DEBUG
hkdebug("kdesimplegrid::if ((datasource()!=NULL)&&(column()!=NULL))");
#endif

 	if (datasource()->is_enabled()==true)
 	    {
		repaint(false);
	   if(hheader!=0)
		  {
   			hheader->set_rownumbers(p_grid->columnscount());
   			for(unsigned int count=0;count<p_grid->columnscount();count++)
   			hheader->setLabel(hheader->mapToLogical(count),
      (p_grid->gridcolumn(count)->displayname().length()==0?p_grid->gridcolumn(count)->columnname().c_str()
   								   :p_grid->gridcolumn(count)->displayname().c_str()),
      p_grid->gridcolumn(count)->columnwidth());
		  }

 		setEnabled(true);
 		//setNumRows(datasource()->max_rows());
 		if(datasource()->columns()!=NULL)setNumCols(p_grid->columnscount());//datasource()->columns()->size());
 		setEnabled(true);
 		setNumRows(datasource()->max_rows()+(datasource()->is_readonly()?0:1));

           if (vheader!=0)
        	{
          	   vheader->set_rownumbers(datasource()->max_rows()+(datasource()->is_readonly()?0:1) );
          	   vheader->set_actualcell(0);
          	   vheader->set_asterics(!datasource()->is_readonly());
          	   vheader->repaint(false);
		}
	p_actualcell_y=0;

	  }
 	else  //disable
 	{
 	 setNumRows(1);
 	 setNumCols(1);
 	 repaint(false);
 	 setEnabled(false);
         return;
 	}
 }
 else
   {
     setEnabled(false);
     //setText("");
   }

	repaint(false);


}



bool hk_kdesimplegrid::datasource_enable(void)
{
#ifdef HK_DEBUG
hkdebug("hk_kdesimplegrid::datasource_enable");
#endif
  hk_dsvisible::datasource_enable();

        emit row_number_changes();
       gotocell(p_actualcell_x,p_actualcell_y);
       blinkcursorslot();
	emit datasource_enables();

  return true;
};

bool hk_kdesimplegrid::datasource_disable(void)
{
#ifdef HK_DEBUG
hkdebug("hk_kdesimplegrid::datasource_disable");
#endif
       // hk_dsgrid::datasource_disable();
        setTopCell(0);
	widget_specific_enable_disable();
		  if ((hheader!=0)) hheader->set_rownumbers(1);
/*		    for(int i=0;i<hheader->count();i++)
  		     {
  		       if (i==0)
  		           hheader->setLabel(i,"",default_cellwidth);
  		       else hheader->setLabel(i,"",0);
  		     }
*/        setNumCols(1);
 	setNumRows(1);
        repaint(false);
 	setEnabled(false);
        if (vheader!=0)
          {
        	vheader->set_rownumbers(1);
        	vheader->set_actualcell(0);
        	vheader->repaint(false);
          }

        emit row_number_changes();
        emit datasource_disables();
return true;
}

bool hk_kdesimplegrid::columns_new_created()
{
#ifdef HK_DEBUG
hkdebug("hk_kdesimplegrid::columns_new_created");
#endif
	bool r=hk_dsvisible::columns_new_created();
	if (p_grid->gridcolumn(p_actualcell_x)->column()!=NULL)
	p_actualcelltext=p_grid->gridcolumn(p_actualcell_x)->column()->asstring_at(p_actualcell_y).c_str();
   	emit new_columns();
	return r;

}



void hk_kdesimplegrid::set_columnwidth(int col,int,int newwidth)
{
#ifdef HK_DEBUG
hkdebug("hk_kdesimplegrid::set_columnwidth ",col);
#endif
    if (hheader!=NULL) p_grid->set_gridcolumnwidth((unsigned)hheader->mapToActual(col),newwidth);
    else    p_grid->set_gridcolumnwidth((unsigned)col,newwidth);
    updateTableSize();
    repaint(true);
}


bool hk_kdesimplegrid::widget_specific_row_change(void)
{
#ifdef HK_DEBUG
hkdebug("hk_kdesimplegrid::widget_specific_row_change");
#endif
  offset=0;cursorpos=0;markanchor=0;markdrag=0;
if (p_grid->gridcolumn(p_actualcell_x)->columntype()==hk_dsgridcolumn::columnbool) p_cursoron=true;
else p_cursoron=false;
  p_blinktimer.stop();
  updateCell(p_actualcell_y,p_actualcell_x,false);
   if (!p_rowcolupdate)
          {
             #ifdef HK_DEBUG
             hkdebug("!rowcolupdate");
             #endif

            if (datasource()!=NULL) gotocell(p_actualcell_x,datasource()->row_position());
          }
   if (vheader!=0)vheader->set_actualcell(p_actualcell_y);
   if (datasource()!=0)
     {
      long int pos=datasource()->row_position();
//      int loffset= (datasource()->mode()==mode_insertrow)?2:1;
      if (pos>=lastRowVisible())
              setTopCell(topCell()+pos-lastRowVisible()+1);
      if (pos<topCell())setTopCell(pos);
      //p_actualcell_y=pos;
     }
   emit actual_row_changes();
   if (!p_rowcolupdate) blinkcursorslot();
   return true;
}

void hk_kdesimplegrid::set_cellheight(int h)
{
#ifdef HK_DEBUG
hkdebug("hk_kdesimplegrid::set_cellheight: ",h);
#endif

 	if(h>5)p_cellheight=h;
 	else p_cellheight=5;
 	setCellHeight(p_cellheight);
        if (vheader!=0)
           {
             vheader->set_cellheight(p_cellheight);
             if (datasource()!=NULL)datasource()->goto_row(vheader->actualcell());
             vheader->repaint(false);
           }
            updateTableSize();

        repaint(false);
}

void hk_kdesimplegrid::set_cellheight(int,int,int newsize)
{
#ifdef HK_DEBUG
hkdebug("hk_kdesimplegrid::set_cellheight init,int,int");
#endif
 set_cellheight(newsize);
}


int hk_kdesimplegrid::cellHeight(void)
{
#ifdef HK_DEBUG
hkdebug("hk_kdesimplegrid::set_cellheight");
#endif
 	return p_cellheight;
}


void hk_kdesimplegrid::set_headers(hk_hheader* horizontal,hk_vheader* vertical,QLabel* statusbar)
{
#ifdef HK_DEBUG
hkdebug("hk_kdesimplegrid::set_headers");
#endif
 	hheader=horizontal;
 	vheader=vertical;
	p_statusbar=statusbar;
 	if (hheader!=0)
 	  {
 	     connect(horizontalScrollBar(),SIGNAL(valueChanged(int)),hheader,SLOT(setOffset(int)));
   	     connect(hheader ,SIGNAL(sizeChange(int,int,int)),this,SLOT(set_columnwidth(int,int,int)));
             hheader->setTracking(true);
             hheader->setMovingEnabled(true);
             connect(hheader,SIGNAL(moved(int,int)),this,SLOT(column_moved(int,int)));
             hheader->set_grid(this);

 	  }
 	if (vheader!=0)
 	  {
 	     connect(verticalScrollBar(),SIGNAL(valueChanged(int)),vheader,SLOT(set_offset(int)));
   	     connect(vheader ,SIGNAL(sizeChange(int,int,int)),this,SLOT(set_cellheight(int,int,int)));
             vheader->set_tracking(false);
             vheader->set_movingenabled(false);
             vheader->set_grid(this);
 	  }

}


void hk_kdesimplegrid::column_moved(int from, int to)
{
#ifdef HK_DEBUG
hkdebug("hk_kdesimplegrid::column_moved");
#endif
   p_grid->change_columnposition((unsigned)from,(unsigned) to);
   if (p_grid->gridcolumn(p_actualcell_x)->column()!=NULL)
   {
    if (p_grid->gridcolumn(p_actualcell_x)->column()->has_changed())
   	p_actualcelltext=p_grid->gridcolumn(p_actualcell_x)->column()->changed_data_asstring().c_str();
    else
  	p_actualcelltext=p_grid->gridcolumn(p_actualcell_x)->column()->asstring_at(p_actualcell_y).c_str();
   }

 updateCell(p_actualcell_y,p_actualcell_x,false);

   repaint(false);

}

void hk_kdesimplegrid::gotocell(unsigned int x, unsigned int y)
{
  #ifdef HK_DEBUG
//  hkdebug ("p_actualcelltext: ",p_actualcelltext.latin1());
  #endif
  if (hasFocus()){offset=cursorpos=markanchor=0;markdrag=p_actualcelltext.length(); }
  if (hheader!=NULL)hheader->set_actualcell(x);
 if (vheader!=NULL)vheader->set_actualcell(y);
  offset=0;cursorpos=0;markanchor=0;markdrag=0;
if (p_grid->gridcolumn(p_actualcell_x)->columntype()==hk_dsgridcolumn::columnbool) p_cursoron=true;
else p_cursoron=false;
  p_blinktimer.stop();
  updateCell(p_actualcell_y,p_actualcell_x,false);
  store_changed_celldata();
  p_actualcell_x=x;
  p_actualcell_y=y;
  if (p_grid->gridcolumn(x)!=NULL)if(p_grid->gridcolumn(x)->column()!=NULL)
  		{
  		 if (p_grid->gridcolumn(x)->column()->has_changed())
  		 	p_actualcelltext=p_grid->gridcolumn(x)->column()->changed_data_asstring().c_str();
  		 else
		 {
  			if (datasource()->mode()==hk_datasource::mode_insertrow
			    && !p_grid->gridcolumn(x)->column()->has_changed())

			{
			    p_actualcelltext=p_grid->gridcolumn(x)->defaultvalue().c_str();
			}
			else
			 p_actualcelltext=p_grid->gridcolumn(x)->column()->asstring_at(y).c_str();
		 }
		if (p_statusbar!=NULL) p_statusbar->setText(p_grid->gridcolumn(x)->column()->is_readonly()?hk_translate("readonly").c_str():hk_translate("readwrite").c_str());

		}
	else
	{
		if (p_statusbar!=NULL) p_statusbar->setText("");
	}
  if (hasFocus()){offset=cursorpos=markanchor=0;markdrag=strlen(p_actualcelltext); }
  blinkcursorslot();
}



void hk_kdesimplegrid::before_row_change(void)
{
  #ifdef HK_DEBUG
  hkdebug ("hk_kdesimplegrid::before_row_change");

  #endif
  store_changed_celldata();
}
void hk_kdesimplegrid::store_changed_celldata(void)
{
  #ifdef HK_DEBUG
  hkdebug ("hk_kdesimplegrid::store_changed_celldata");

  #endif

  if (p_datachanged)
     {
  #ifdef HK_DEBUG
  hkdebug ("store_changed_celldata");

  #endif
       if (p_grid->gridcolumn(p_actualcell_x)->column()!=NULL)
           p_grid->gridcolumn(p_actualcell_x)->column()->set_asstring(p_actualcelltext.latin1());
       p_datachanged=false;
     }
     else
     {
  #ifdef HK_DEBUG
  hkdebug ("!store_changed_celldata");

  #endif

     }

}



void hk_kdesimplegrid::widget_specific_insert_mode(void)
{
  #ifdef HK_DEBUG
  hkdebug ("hk_kdesimplegrid::insert_mode");
  #endif
  if (datasource()==NULL)return;
  unsigned int y=datasource()->max_rows();
   gotocell(p_actualcell_x,y);
   for(unsigned int k=0;k<p_grid->columnscount();k++)updateCell(y,k,false);
   for(unsigned int k=0;k<p_grid->columnscount();k++)updateCell(y-2,k,false);
   if (y>(unsigned)lastRowVisible()) setTopCell(topCell()+y-lastRowVisible());
   setTopCell(numRows()-lastRowVisible()+topCell());

   blinkcursorslot();
}

void hk_kdesimplegrid::nextcell(void)
{
  #ifdef HK_DEBUG
  hkdebug ("hk_kdesimplegrid::nextcell");
  #endif
  unsigned int x=p_actualcell_x;
  x++;
  if (x<p_grid->columnscount())
  {
    gotocell(x,p_actualcell_y);
    if ((unsigned)lastColVisible()<x)
      setLeftCell(leftCell()+1);
  }
  else
  {
   if (datasource()!=0)
      {
         if(((signed)datasource()->row_position()<(signed)datasource()->max_rows()-1)
              &&(datasource()->mode()==hk_datasource::mode_normal) )
           {
            datasource()->goto_next();
            gotocell(0,p_actualcell_y);
            blinkcursorslot();
            setLeftCell(0);
           }
         else
         {

          datasource()->setmode_insertrow();
          gotocell(0,p_actualcell_y);
          blinkcursorslot();
          setLeftCell(0);

         }
      }
  }

}

void hk_kdesimplegrid::previouscell(void)
{
  #ifdef HK_DEBUG
  hkdebug ("hk_kdesimplegrid::previouscell");
  #endif
  int  x=p_actualcell_x;
  x--;
  if (x>=0)
    {
      gotocell(x,p_actualcell_y);
      blinkcursorslot();
      if (x<leftCell()) setLeftCell(x);
    }
  else
  {
     if (datasource()!=0)
     {
        if (datasource()->row_position()!=0)
        {
            datasource()->goto_previous();
            gotocell(p_grid->columnscount()-1,p_actualcell_y);
     	    blinkcursorslot();
     	    if ((unsigned)lastColVisible()<p_grid->columnscount()-1)
            setLeftCell(p_grid->columnscount()-1);
        }

     }
  }
}



void hk_kdesimplegrid::widget_specific_row_added()
{
  #ifdef HK_DEBUG
  hkdebug ("hk_kdesimplegrid::widget_specific_row_added");
  #endif
  if (datasource()!=0)
     {
       setNumRows(datasource()->max_rows()+(datasource()->is_readonly()?0:1));
        if (vheader!=0)
       	   vheader->set_rownumbers(datasource()->max_rows()+(datasource()->is_readonly()?0:1) );
     }
  else setNumRows(1);
}

void hk_kdesimplegrid::widget_specific_row_deleted()
{
  #ifdef HK_DEBUG
  hkdebug ("hk_kdesimplegrid::widget_specific_row_deleted");
  #endif
  if (datasource()!=0)
     {
       setNumRows(datasource()->max_rows()+(datasource()->is_readonly()?0:1));
        if (vheader!=0)
       	   vheader->set_rownumbers(datasource()->max_rows()+(datasource()->is_readonly()?0:1) );
     }
  else setNumRows(1);
  repaint(true);

}


void hk_kdesimplegrid::rowselector_clickedslot(void)
{
  #ifdef HK_DEBUG
  hkdebug ("hk_kdesimplegrid::rowselector_clickedslot");
  #endif
  if (!hasFocus()) setFocus();

}

void hk_kdesimplegrid::set_rowselector(hk_kderowselector* s)
{
  #ifdef HK_DEBUG
  hkdebug ("hk_kdesimplegrid::set_rowselector");
  #endif

        connect(s,SIGNAL(rowselector_clicked()),this,SLOT(rowselector_clickedslot()));


}


void hk_kdesimplegrid::blinkcursorslot(void)
{
  #ifdef HK_DEBUG
  hkdebug ("hk_kdesimplegrid::blinkcursorslot");
  #endif
if (hasFocus()||p_cursoron)
	{
	 p_cursoron=!p_cursoron;
//	 if (!pmdirty&&p_cursorrect.isValid()) updateCell(p_actualcell_y,p_actualcell_x,false);
	 updateCell(p_actualcell_y,p_actualcell_x,false);
	 if (!hasFocus()) p_blinktimer.stop();
	 else if(!p_blinktimer.isActive()) p_blinktimer.start(p_blinkspeed);
	}
}

void hk_kdesimplegrid::blinkon()
{
    if ( !hasFocus() )
	return;

    p_blinktimer.start( p_cursoron?QApplication::cursorFlashTime() / 2 : 0, TRUE );
    blinkcursorslot();
}


void hk_kdesimplegrid::focusInEvent(QFocusEvent * f)
{
#ifdef HK_DEBUG
hkdebug("hk_kdesimplegrid::focusInEvent");
#endif
	QTableView::focusInEvent(f);
 p_cursoron=false;
 if (p_grid->gridcolumn(p_actualcell_x)!=NULL)
 	if (p_grid->gridcolumn(p_actualcell_x)->columntype()==hk_dsgridcolumn::columnbool) p_cursoron=true;
	pmdirty=true;
	blinkcursorslot();

}


void hk_kdesimplegrid::focusOutEvent(QFocusEvent * f)
{
#ifdef HK_DEBUG
hkdebug("hk_kdesimplegrid::focusOutEvent");
#endif

  offset=0;cursorpos=0;markanchor=0;markdrag=0;
  p_cursoron=false;
 if (p_statusbar!=NULL) p_statusbar->setText("");
 if (p_grid->gridcolumn(p_actualcell_x)!=NULL)
	if (p_grid->gridcolumn(p_actualcell_x)->columntype()==hk_dsgridcolumn::columnbool) p_cursoron=true;
  p_blinktimer.stop();
  updateCell(p_actualcell_y,p_actualcell_x,false);

	QTableView::focusOutEvent(f);
	if (p_cursoron)blinkcursorslot();
	if (p_datachanged)
	  {
	    store_changed_celldata();
	    if (datasource()!=0) datasource()->store_changed_data();
	    repaint(false); //???????????????
	  }
}


void hk_kdesimplegrid::keyPressEvent(QKeyEvent* e)
{
  #ifdef HK_DEBUG
  hkdebug ("hk_kdesimplegrid::keyPressEvent");
  #endif
if (p_grid->gridcolumn(p_actualcell_x)->columntype()==hk_dsgridcolumn::columnbool)
{
if (key_pressevent_navigation(e))return;
	if ((e->key()==Key_Space)&&(p_grid->gridcolumn(p_actualcell_x)->column()!=NULL))
	  {
	   if (p_grid->gridcolumn(p_actualcell_x)->column()->has_changed())
	         p_grid->gridcolumn(p_actualcell_x)->column()->set_asbool(!p_grid->gridcolumn(p_actualcell_x)->column()->changed_data_asbool());
	    else
                p_grid->gridcolumn(p_actualcell_x)->column()->set_asbool(!p_grid->gridcolumn(p_actualcell_x)->column()->asbool());
          }

             blinkon();


}
else
{
if (key_pressevent_edit(e))return;
if (key_pressevent_navigation(e))return;
if (e->state()&ControlButton)
   {
       switch(e->key())
       {
        case Key_Insert:
        case Key_C: 	 if(has_markedtext())copy();
        		return;
	case Key_F: p_grid->find_clicked();
			return;
	default:;

        }
   }
   e->ignore();
}
}
bool hk_kdesimplegrid::key_pressevent_navigation(QKeyEvent* e)
{
  #ifdef HK_DEBUG
  hkdebug ("hk_kdesimplegrid::key_pressevent_navigation");
  #endif
  bool r=true;
if (e->state()&ControlButton)
   {
       switch(e->key())
       {
        case Key_A:
        case Key_Left:	previouscell();
        		break;
        case Key_E:
        case Key_Right:	nextcell();
        		break;
        case Key_Return: previouscell();
        		break;
        case Key_Up: 	if(datasource()!=0)
        		{
        		   store_changed_celldata();
        		   if (datasource()->mode()==hk_datasource::mode_insertrow) datasource()->goto_row(datasource()->max_rows()-1);
        		   else   datasource()->goto_previous();
        		}
        		break;
        case Key_Down: if(datasource()!=0)
         		{
         		   store_changed_celldata();
         		   if ((signed)p_actualcell_y<(signed)datasource()->max_rows()-1)datasource()->goto_next();
         		   else datasource()->setmode_insertrow();
         		}
        		break;
        default:	r=false;
       }
   }
else
     {
       switch (e->key())
       {
        case Key_Left:  cursor_left(e->state() & ShiftButton);
        		break;
        case Key_Right:	cursor_right(e->state() & ShiftButton);
        		break;
        case Key_Home:	home(e->state()&ShiftButton);
        		break;
        case Key_End:	end(e->state()&ShiftButton);
        		break;
        case Key_Return: nextcell();
        		break;
        default: r=false;
       }
    }

  return r;

}


bool hk_kdesimplegrid::key_pressevent_edit(QKeyEvent* e)
{
  #ifdef HK_DEBUG
  hkdebug ("hk_kdesimplegrid::key_pressevent_edit");
  #endif
  bool r=true;
  if (	e->ascii()>=32&&
	e->key()!=Key_Delete &&
	e->key()!=Key_Backspace
    )
	{
	 QString t=e->text();
	 if (!t.isEmpty()) insert(t);
	 return r;
	}

if (e->state()&ControlButton)
   {
       switch(e->key())
       {
        case Key_D: 	 del();
        		break;
        case Key_H:	backspace();
        		break;
        case Key_V: 	 insert(QApplication::clipboard()->text());
        case Key_X: 	 if(has_markedtext())copy();
        		del();
        		break;

        default:	r=false;
       }
   }
else
   {
     if (e->state()&ShiftButton)
     {
       switch (e->key())
       {
        case Key_Insert: insert(QApplication::clipboard()->text());
                 	 if(has_markedtext())copy();
        	  	 del();
        		 break;
        case Key_Delete: if(has_markedtext())
        		  {
        		    copy();
        		    del();
        		  }
        		break;
        default:r=false;
       }
     }
     else
     {
       switch (e->key())
       {
        case Key_Delete:del();
        		break;
        case Key_Backspace: backspace();
        		break;
        case Key_Escape: escape();
        		 break;
        default: r=false;
       }
    }
   }
return r;
}

void hk_kdesimplegrid::cursor_left( bool mark, int steps )
{
    cursor_right( mark, -steps );
}

void hk_kdesimplegrid::cursor_right( bool mark, int steps )
{
    int cp = cursorpos + steps;
    cp = QMAX( cp, 0 );
    cp = QMIN( cp, (int)p_actualcelltext.length() );
    if ( cp == cursorpos )
    {
	if ( !mark )
	    deselect();
    }
      else
      if ( mark )
      {
	newmark( cp,false );
	blinkon();
      }
      else
      {
	set_cursorposition( cp );
	set_selection( cp, 0 );
      }
}

void hk_kdesimplegrid::set_cursorposition( int newPos )
{
    if ( newPos == cursorpos )
	return;
    newPos = QMIN( newPos, (int)p_actualcelltext.length() );
    newPos = QMAX( newPos, 0 );
    int b, e;
    b = QMIN( newPos, cursorpos );
    e = QMAX( newPos, cursorpos );
    cursorpos = newPos;
    blinkon();
//    repaintArea( b, e );
}


bool hk_kdesimplegrid::settext(QString newtext,int newpos,int newmarkanchor, int newmarkdrag)
{
//original QLineedit::validateandset
    QString t( newtext );
    for ( uint i=0; i<t.length(); i++ ) {
	if ( t[(int)i] < ' ' )  // unprintable/linefeed becomes space
	    t[(int)i] = ' ';
    }
    bool tc = ( t != p_actualcelltext );

    if ( newmarkdrag != markdrag ||
	 newmarkanchor != markanchor ||
	 newpos != cursorpos ||
	 tc ) {
	int minP = QMIN( cursorpos, minmark() );
	int maxP = QMAX( cursorpos, maxmark() );

	cursorpos = newpos;
	markanchor = newmarkanchor;
	markdrag = newmarkdrag;

	minP = QMIN( minP, QMIN( cursorpos, minmark() ) );
	int i = 0;
	while( i < minP && t[i] == p_actualcelltext[i] )
	    i++;
	minP = i;

	maxP = QMAX( maxP, QMAX( cursorpos, maxmark() ) );
	if ( fontMetrics().width( t ) < fontMetrics().width(p_actualcelltext ) )
	    maxP = t.length();
	p_actualcelltext = t;

	if ( cursorpos < (int)p_actualcelltext.length() && maxP < (int)p_actualcelltext.length() )
	    maxP = p_actualcelltext.length();
       updateCell(p_actualcell_x,p_actualcell_y,false);
       p_cursoron=false;
       blinkcursorslot();

   }
  emit text_changed( p_actualcelltext );
  p_datachanged=true;

    return true;

}


void hk_kdesimplegrid::update_offset()
{ // must not call repaint() - paintEvent() calls this
    if ( !isVisible() )
    {
	offsetdirty = TRUE;
	return;
    }
    offsetdirty = FALSE;
    QFontMetrics fm = fontMetrics();
    int textWidth = fm.width( p_actualcelltext )+4;
    int w = cellWidth(p_actualcell_x);
    int old = offset;

    if ( textWidth > w )
    {
	// may need to scroll.
	QString dt = p_actualcelltext;
	dt += QString::fromLatin1( "  " );
	dt = dt.left( cursorpos + 2 );
	if ( cursorpos < 3 )
	    offset = 0;
	else if ( fm.width( dt.left( cursorpos - 2 ) ) + offset < 0 )
	    offset = -fm.width( dt.left( cursorpos - 2 ) );
	else if ( fm.width( dt ) + offset > w )
	    offset = w - fm.width( dt );
    }
    else
    {
	if ( textWidth < 5 )
	{
	    // nothing is to be drawn.  okay.
	    textWidth = QMIN( 5, w );
	}
    offset = 0;
    }


    if ( old == offset && !pmdirty )
	return;

    pmdirty = TRUE;
}

void hk_kdesimplegrid::insert(const QString& newtext)
{
  #ifdef HK_DEBUG
  hkdebug ("hk_kdesimplegrid::insert");
  #endif
        if (p_grid->gridcolumn(p_actualcell_x)->column()==NULL) return;
        if (p_grid->gridcolumn(p_actualcell_x)->column()->is_readonly()) return;
        if (p_grid->gridcolumn(p_actualcell_x)->column()->fieldtype()==hk_column::binaryfield) return;
 	QString t(newtext);
 	if (t.isEmpty()) return;

    	for ( int i=0; i<(int)t.length(); i++ )
	  if ( t[i] < ' ' )  // unprintable/linefeed becomes space
	       t[i] = ' ';


	QString test(p_actualcelltext);
 	int cp = cursorpos;
 	if (has_markedtext())
 	{
 	   test.remove(minmark(),maxmark()-minmark());
 	   cp=minmark();
 	}

  	test.insert(cp,t);
 	p_cursoron=false;
 	cp=QMIN(cp+t.length(),(uint)p_maxlen);
 	blinkcursorslot();
 	settext(test,cp,cp,cp);
}

const bool hk_kdesimplegrid::has_markedtext()
{
  return markanchor!=markdrag;
}

void hk_kdesimplegrid::set_selection( int start, int length )
{
    int b, e;
    b = QMIN( markanchor, markdrag );
    e = QMAX( markanchor, markdrag );
    b = QMIN( b, start );
    e = QMAX( e, start + length );
    markanchor = start;
    markdrag = start + length;
    blinkcursorslot();
 }


void hk_kdesimplegrid::deselect()
{
    set_selection( cursorpos, 0 );
}

void hk_kdesimplegrid::newmark( int pos, bool  )
{
    if ( markdrag != pos || cursorpos != pos )
	pmdirty = TRUE;
    markdrag = pos;
    set_cursorposition( pos );
//    if ( c ) copy();
}

void hk_kdesimplegrid::copy()
{
  #ifdef HK_DEBUG
  hkdebug ("hk_kdesimplegrid::copy");
  #endif
  QString t=marked_text();
  if (!t.isEmpty())
     {
        disconnect(QApplication::clipboard(),SIGNAL(dataChanged()),this,0);
        QApplication::clipboard()->setText(t);
        connect(QApplication::clipboard(),SIGNAL(dataChanged()),this,SLOT(clipboard_changed()));

     }

}

QString hk_kdesimplegrid::marked_text()
{
  #ifdef HK_DEBUG
  hkdebug ("hk_kdesimplegrid::marked_text");
  #endif
   if (markanchor!=markdrag)
     {
      return p_actualcelltext.mid(minmark(),maxmark()-minmark());
     }
   else return 0;
}


void hk_kdesimplegrid::del(void)
{
  #ifdef HK_DEBUG
  hkdebug ("hk_kdesimplegrid::del");
  #endif
  if (p_grid->gridcolumn(p_actualcell_x)->column()==NULL)return;
  if ((p_grid->gridcolumn(p_actualcell_x)->column()->is_readonly())) return;
  if (p_grid->gridcolumn(p_actualcell_x)->column()->fieldtype()==hk_column::binaryfield) return;

 	QString test(p_actualcelltext.copy());
 	if (has_markedtext())
 	{
 	    test.remove(minmark(),maxmark()-minmark());
 	    settext(test,minmark(),minmark(),minmark());
 	}
 	else
 	{
 	   if (cursorpos!=(int)strlen(p_actualcelltext))
 	    {
 	       test.remove(cursorpos,1);
 	       settext(test,minmark(),minmark(),minmark());
 	    }
 	}
 	p_cursoron=false;
 	blinkcursorslot();
}

void hk_kdesimplegrid::backspace(void)
{
  #ifdef HK_DEBUG
  hkdebug ("hk_kdesimplegrid::backspace");
  #endif
   if (has_markedtext()) del();
   else
   {
      if (cursorpos>0)
      {
         cursor_left(false);
         del();
      }

   }
}

void hk_kdesimplegrid::escape()
{
  #ifdef HK_DEBUG
  hkdebug ("hk_kdesimplegrid::escape");
  #endif
    p_actualcelltext=p_grid->gridcolumn(p_actualcell_x)->column()->asstring_at(p_actualcell_y).c_str();
  if (hasFocus()){offset=cursorpos=markanchor=0;markdrag=strlen(p_actualcelltext); }
  p_datachanged=false;
}

void hk_kdesimplegrid::home(bool mark)
{
  #ifdef HK_DEBUG
  hkdebug ("hk_kdesimplegrid::home");
  #endif
    cursor_right( mark, -cursorpos );
}

void hk_kdesimplegrid::end(bool mark)
{
  #ifdef HK_DEBUG
  hkdebug ("hk_kdesimplegrid::end");
  #endif
    cursor_right( mark, p_actualcelltext.length()-cursorpos );

}


void hk_kdesimplegrid::mousePressEvent(QMouseEvent* m)
{
  #ifdef HK_DEBUG
  hkdebug ("hk_kdesimplegrid::mousePressEvent");

  #endif
  if (datasource()==NULL)return;
     if (m->button()==RightButton)
   {
    QPopupMenu* popup=new QPopupMenu(this);
    int id[8];
/*	id[ 0 ] = popup->insertItem( hk_translate( "Cut" ) );
	id[ 1 ] = popup->insertItem( hk_translate( "Copy" ) );
	id[ 2 ] = popup->insertItem( hk_translate( "Paste" ) );
	popup->insertSeparator();
*/	id[ 3 ] = popup->insertItem( hk_translate( "Delete row" ).c_str() );
	popup->insertSeparator();
	id[ 4 ] = popup->insertItem( hk_translate( "Define columns" ).c_str() );
	id[ 5 ] = popup->insertItem( hk_translate( "Find" ).c_str() );
	int r = popup->exec( m->globalPos() );

      if (r==id[3]&&datasource()!=NULL)datasource()->delete_actualrow();
      else if (id[0]==r)
      		{
		   copy();del();
		}
      else if (id[1]==r)copy();
      else if (id[2]==r)insert(QApplication::clipboard()->text());
      else if (id[4]==r)
                       {
			 hk_kdegridcolumndialog* dialog=new hk_kdegridcolumndialog(p_grid,0,true);

			 int res=dialog->exec();
			 if (res==QDialog::Accepted)
			 {
			   list<hk_string> p_l;
/*			   int c =dialog->rightbox->childCount();
			   for (int k=0;k<c;k++)
			   {

			    p_l.insert(p_l.end(),dialog->rightbox->text(k).latin1());
			   }
*/
			 QListViewItem* item=dialog->rightbox->firstChild();
			 while (item!=NULL)
			 {
			 p_l.insert(p_l.end(),item->text(0).latin1());
    			 item=item->nextSibling();
			 }

			 p_grid->set_gridcolumns(p_l);


			 item=dialog->rightbox->firstChild();
			 unsigned int k=0;
			 while (item!=NULL)
			 {
			 hk_dsgridcolumn* gc=p_grid->gridcolumn(k);
			 if (gc!=NULL)
			  {
			    if(item->text(1).isEmpty()) gc->reset_default();
			    else 	gc->set_defaultvalue(item->text(1).latin1());
			  }
    			 item=item->nextSibling();k++;
			 }
			 widget_specific_enable_disable();
			 }
			 delete dialog;
		       }
      else if (id[5]==r)p_grid->find_clicked();
    delete popup;
    return;
   }

	 int x=findCol(m->x());
	 int y=findRow(m->y());
	p_rowcolupdate=true;
	store_changed_celldata();
	if ((x==(signed)p_actualcell_x)&&(y==(signed)p_actualcell_y))
	{
   	   int cp;
	   colXPos(p_actualcell_x,&cp);
           p_newcellenter=false;
  #ifdef HK_DEBUG
  hkdebug ("mousePressEvent vor xpostocursorpos");

  #endif
	   cursorpos=xPosToCursorPos(m->pos().x()-cp);
  #ifdef HK_DEBUG
  hkdebug ("mousePressEvent nach xpostocursorpos");

  #endif
	   markanchor=cursorpos;
	   newmark(markanchor,false);
	}
	else
	{

	if ((y!=(signed)p_actualcell_y)&&(datasource()!=0))
	    {
	        #ifdef HK_DEBUG
	        hkdebug("gotorow");
	        #endif
		if ((y>-1)&&(x>-1))datasource()->goto_row(y);
	    }

	if ((y>-1)&&(x>-1))
		{
		   p_newcellenter=true;
		   gotocell(x,y);
		   if ((unsigned)y==datasource()->max_rows()&&(datasource()->mode()!=hk_datasource::mode_insertrow))
		     {
		      datasource()->setmode_insertrow();
		     }

		}


	}
if (p_grid->gridcolumn(p_actualcell_x)->columntype()==hk_dsgridcolumn::columnbool)
{
	if ((p_grid->gridcolumn(p_actualcell_x)->column()!=NULL))
	  {
	   if (p_grid->gridcolumn(p_actualcell_x)->column()->has_changed())
	         p_grid->gridcolumn(p_actualcell_x)->column()->set_asbool(!p_grid->gridcolumn(p_actualcell_x)->column()->changed_data_asbool());
	    else
                p_grid->gridcolumn(p_actualcell_x)->column()->set_asbool(!p_grid->gridcolumn(p_actualcell_x)->column()->asbool());
          }


}

   blinkcursorslot();
     p_rowcolupdate=false;

}

void hk_kdesimplegrid::mouseMoveEvent(QMouseEvent* m)
{
  #ifdef HK_DEBUG
  hkdebug ("hk_kdesimplegrid::mouseMoveEvent");
  #endif
//   int x=findCol(m->x());
   int cp;
   colXPos(p_actualcell_x,&cp);
   int rel_pos=m->pos().x()-cp;
   if ((rel_pos<4)||(rel_pos>cellWidth(p_actualcell_x)-4))
   {//scroll and mark text which is not visible in the moment
    p_scrollleft=(rel_pos<4);
    if (!p_cellscrolling)
    { //start scrolling and marking
       p_cellscrolling=true;
       if (p_scrollleft) newmark(offset,false);
       else newmark(lastcharvisible(),false);
       p_cellscrolltimer.start(p_cellscrollspeed);
    }
    else
    {//continue scrolling and marking
      if (p_scrollleft)
      {
         int steps=-(rel_pos+4)/15+2;
         cursor_left(true,steps);
      }
      else
      {
         int steps=(rel_pos-cellWidth(p_actualcell_x)+4)/15+2;
         cursor_right(true,steps);
      }

    }
   }
    else
    { //no scrolling => set cursor at mouse position
    p_cellscrolling=false;
  #ifdef HK_DEBUG
  hkdebug ("mouseMoveEvent vor xpostocursorpos");
  #endif
   int mousepos=offset+ xPosToCursorPos(m->pos().x()-cp);
  #ifdef HK_DEBUG
  hkdebug ("mouseMoveEvent nach xpostocursorpos");
  #endif
   newmark(mousepos,false);

    }


}

void hk_kdesimplegrid::mouseReleaseEvent(QMouseEvent* m)
{
  #ifdef HK_DEBUG
  hkdebug ("hk_kdesimplegrid::mouseReleaseEvent");
  #endif
  if (!p_newcellenter)
  {
   	int cp;
   	colXPos(p_actualcell_x,&cp);
  #ifdef HK_DEBUG
  hkdebug ("mouseReleaseEvent vor xpostocursopos");
  #endif
  	int mousepos=offset+ xPosToCursorPos(m->pos().x()-cp);
  #ifdef HK_DEBUG
  hkdebug ("mouseReleaseEvent nach xpostocursopos");
  #endif
   	newmark(mousepos,false);
   }
   p_cellscrolling=false;

}


int hk_kdesimplegrid::lastcharvisible()
{
    int tDispWidth = cellWidth(p_actualcell_x)-p_margin;
    return xPosToCursorPos( tDispWidth );
}



void hk_kdesimplegrid::clipboard_changed(void)
{
  #ifdef HK_DEBUG
  hkdebug ("hk_kdesimplegrid::clipboard_changed");
  #endif
        disconnect(QApplication::clipboard(),SIGNAL(dataChanged()),this,SLOT(clipboard_changed()));
        markdrag=markanchor=cursorpos;
  	updateCell(p_actualcell_y,p_actualcell_x,false);


}

void hk_kdesimplegrid::cellscrollslot(void)
{
  #ifdef HK_DEBUG
  hkdebug ("hk_kdesimplegrid::cellscrollslot");
  #endif
  if (!hasFocus()||!p_cellscrolling)
    p_cellscrolltimer.stop();
  else
    if (p_scrollleft)cursor_left(true);
    else cursor_right(true);
}

int hk_kdesimplegrid::totalHeight()
{
	return cellHeight()*numRows();

}

bool hk_kdesimplegrid::widget_specific_coordinates(unsigned int px,unsigned int py,unsigned int pwidth,unsigned int pheight)
{
   //bool res=hk_dsgrid::widget_specific_coordinates(px,py,pwidth,pheight);
  #ifdef HK_DEBUG
  hkdebug ("hk_kdesimplegrid::widget_specific_coordinates");
  #endif
   QWidget* wid=parentWidget();
   if  (wid!=NULL)
      {
	wid->setGeometry(px,py,pwidth,pheight);

      }
   return true;
}

hk_column* hk_kdesimplegrid::actual_column(void)
{

  return p_grid->gridcolumn(p_actualcell_x)->column();
}
