// ****************************************************************************
// copyright (c) 2000,2001 Horst Knorr <hk_classes@knorrnet.de>
// This file is part of the hk_kdeclasses library.
// This file may be distributed and/or modified under the terms of the
// GNU General Public License version 2 as published by the Free Software
// Foundation and appearing in the file COPYING included in the
// packaging of this file.
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
// ****************************************************************************

#ifndef HK_KDESIMPLEGRID
#define HK_KDESIMPLEGRID
#include <hk_dsvisible.h>
#include <qtableview.h>
#include <qheader.h>
#include <qtimer.h>
#include <hk_kderowselector.h>
class hk_kdesimpleform;
class hk_kdesimplegrid;
class hk_kdegrid;
class QLabel;
class hk_column;
/**
*
*@short KDE Widget to display data in a grid.
*@version $Revision: 1.10 $
*@author Horst Knorr (hk_classes@knorrnet.de)

*@internal
*/

class hk_header:public QHeader
{
 public:
 hk_header(QWidget* w=0);
 void set_rownumbers(long unsigned int n);
 int actualcell(void){return p_actualcell;}
 void set_actualcell(long unsigned int a){p_actualcell=a;repaint(true);}
 void set_grid(hk_kdesimplegrid* g);
 void set_asterics(bool a);


 protected:
 bool p_asterics;
 long unsigned int p_rows;
 int p_height;
 long unsigned int p_actualcell;
 hk_kdesimplegrid* p_grid;

 private:

};
/**
*@internal
*/
class hk_vheader:public QTableView ,public hk_class
{Q_OBJECT
 public:
 hk_vheader(QWidget* w);
 void set_rownumbers(unsigned long int i);
 void set_actualcell(long unsigned int a){p_actualcell=a;repaint(true);}
 void set_grid(hk_kdesimplegrid* g);
 void set_asterics(bool a);
 void set_cellheight(int h);
 void set_cellwidth(int h);
 int actualcell(void){return p_actualcell;}
 void set_tracking( bool enable );
 bool	tracking() {return p_tracking;}
 void	set_movingenabled( bool );


signals:
    void	sizeChange( int section, int oldSize, int newSize );
protected slots:
void 	set_offset(int);
 protected:
 virtual    void 	paintCell(QPainter* p, int row, int col);
 virtual    void	mousePressEvent( QMouseEvent * );
 virtual    void	mouseReleaseEvent( QMouseEvent * );
 virtual    void	mouseMoveEvent( QMouseEvent * );
virtual int	totalHeight(void);


 private:
 void init(int rows);
// void handle_resize( int index, int s, bool final );
 int ppos(int i);
 long unsigned int p_actualcell;
 bool p_asterics;
 hk_kdesimplegrid* p_grid;
 bool p_tracking;
 bool p_moving;
 bool p_splitcursor;
 bool p_mousekeypressed;
 int p_height;
 int p_resizestartposition;
 int p_resizerow;
 int p_newsize;
 int p_originalsize;

};

/**
*@internal
*/

class hk_hheader:public hk_header
{
 public:
 hk_hheader(QWidget* w);
 protected:
// virtual void paintCell(QPainter* p,int row,int col);
} ;




/**
*
*@short  KDE widget shows data in a grid with columnheaders.
*@version 0.2
*@author Horst Knorr (hk_classes@knorrnet.de)
*@internal
*
*This widget shows data of a datasource in a grid.
*If the data is not readonly you can edit the data just by typing
*in the cell.
*Usually you dont have to use this widget, use @ref hk_kdegrid instead.
*This widget has neither column, row headers nor a rowselector.
*You can navigate through the cells via mouse clicks or keyboard.
*
* Keyboard keys:
*
*===============
*
*Navigation between cells: <ctrl> combined with the cursor keys and
*Enter key (next cell) and <ctrl>+Enter key (previous cell).

*Mark text: <shift>+ cursor right or cursor left key.

*Navigation in a cell: cursor keys.

*copy: <ctrl>+c

*paste: <ctrl>+v

*search: <ctrl>+f

*/

class hk_kdesimplegrid :public QTableView, public hk_dsvisible
{ Q_OBJECT
  friend class hk_kdegrid;
	public:
	 hk_kdesimplegrid(hk_kdegrid* grid, QWidget* wid,hk_kdesimpleform* form=NULL);
virtual  ~hk_kdesimplegrid(void);
/**
*@return the width of a column
*param col number of the column
*/
	int 	cellWidth(int col);
/**
* returns the cellheight. All cells have the same height.
*/
	int 	cellHeight(void);
/**
* takes over the control of a horizontal and a vertical header.
*/
 	void 	set_headers(hk_hheader* horizontal,hk_vheader* vertical,QLabel* statusbar);
/**
* connects to the signal rowselector_clicked.
*/
 	void  	set_rowselector(hk_kderowselector* s);



	signals:
/**
* signal will be emitted whenever the datasource creates new columns (i.e. when enabling).
*/
	void 	new_columns(void);
/**
* signal will be emitted whenever the datasource disables.
*/
	void 	datasource_disables(void);
/**
* signal will be emitted whenever the datasource enables.
*/

	void 	datasource_enables(void);
/**
* signal will be emitted whenever the datasource adds or deletes a row . In the moment it will
*just be emitted when enabling/disabling.
*/
	void 	row_number_changes(void);
/**
* signal will be emitted whenever the datasource goes to another row.
*/
	void    actual_row_changes(void);

       void 	text_changed( const QString &);


	public slots:
/**
* sets the new width of a specific column.
*@param col number of the column
*@param newwidth the new width of the column
*/

	void 	set_columnwidth(int col,int,int newwidth);
/**
* sets the height for all columns.
*/

	void 	set_cellheight(int h);

	void 	set_cellheight(int,int,int newsize);
/**
*its possible to change the order of the columns
*@param from position where the column was originally
*@param to target position
*/
	void 	column_moved(int from,int to);
/**
*/
        bool 	settext(QString,int,int,int);
hk_column*      actual_column(void);
	protected:
	void 	paintCell(class QPainter*painter,int y,int x);
	void 	paint_textcell(class QPainter*painter,int y,int x);
	void 	paint_boolcell(class QPainter*painter,int y,int x);
virtual	void 	widget_specific_enable_disable(void);
virtual	bool 	widget_specific_row_change(void);
virtual	void 	widget_specific_row_added(void);
virtual void 	widget_specific_row_deleted(void);
virtual bool 	widget_specific_coordinates(unsigned int px,unsigned int py,unsigned int pwidth,unsigned int pheight);
	bool   	datasource_enable(void);
	bool 	datasource_disable(void);
 	bool 	columns_new_created(void);
 	void 	gotocell(unsigned int x, unsigned int y);
	void 	before_row_change();
virtual	void 	widget_specific_insert_mode(void);
	QString marked_text(void);
	void 	copy(void);
	void 	focusInEvent(QFocusEvent * f);
	void 	focusOutEvent(QFocusEvent * f);
	void 	keyPressEvent(QKeyEvent * );
	bool 	key_pressevent_navigation(QKeyEvent *);
	bool 	key_pressevent_edit(QKeyEvent*);
        void 	mousePressEvent( QMouseEvent * );
virtual void 	mouseMoveEvent(QMouseEvent*);
virtual void 	mouseReleaseEvent(QMouseEvent*);
virtual int 	totalHeight(void);



	private slots:
 	void 	rowselector_clickedslot(void);
	void    blinkcursorslot(void);
    	void 	clipboard_changed();
 	void 	cellscrollslot(void);

	private:
	int 		lastcharvisible();
	void 		blinkon(void);
const	int 		maxmark() ;
const	int 		minmark() ;
    	void 		update_offset();
	void 		insert(const QString& newtext);
const	bool 		has_markedtext(void);

    	void 		cursor_left(bool mark,int steps=1);
    	void 		cursor_right(bool mark,int steps=1);
virtual void 		set_cursorposition( int );
	void 		set_selection( int start, int length );
	void 		deselect(void);
	void 		newmark( int pos, bool c=true);
    	void 		del(void);
    	void 		backspace(void);
    	void 		home(bool mark);
    	void 		end(bool mark);
    	void 		escape();


	int 		p_cellheight;
	int 		p_maxlen;
        hk_hheader*	hheader;
        hk_vheader*	vheader;
	QLabel*		p_statusbar;
        QTimer		p_blinktimer;
        QTimer 		p_cellscrolltimer;
        unsigned int 	p_actualcell_x;
        unsigned int 	p_actualcell_y;
        bool		p_cursoron;
        bool		p_newcellenter;
//        QRect		p_cursorrect;
        bool 		pmdirty;
	bool 		offsetdirty;
        bool 		p_cellscrolling;
        bool 		p_scrollleft;
        QString		p_actualcelltext;
        bool 		p_datachanged;
        int 		p_blinkspeed;
        int 		p_cellscrollspeed;
        int 		offset;
        int 		cursorpos;
        int 		markanchor;
        int 		markdrag;
        int 		p_margin;
	hk_kdegrid*     p_grid;

    	void 		nextcell();
    	void 		previouscell();
    	bool 		p_rowcolupdate;
    	void 		store_changed_celldata(void);
//the following methods are taken from QLineEdit which is copyrighted by Troll Tech
int xPosToCursorPos( int goalx );
int showLastPartOffset(char*s,const QFontMetrics &fm,int width);

};
#endif
