// ****************************************************************************
// copyright (c) 2000,2001 Horst Knorr <hk_classes@knorrnet.de>
// This file is part of the hk_kdeclasses library.
// This file may be distributed and/or modified under the terms of the
// GNU General Public License version 2 as published by the Free Software
// Foundation and appearing in the file COPYING included in the
// packaging of this file.
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
// ****************************************************************************
//$Revision: 1.10 $

#include "hk_kdetable.h"
#include "hk_kdetable.moc"
#include "hk_kdetabledesign.h"
#include "hk_kdegrid.h"
#include <qbuttongroup.h>
#include <qpushbutton.h>
#include <qtable.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qimage.h>
#include <qpixmap.h>
#include "hk_kdetoolbar.h"


/*
 *  Constructs a hk_kdetable which is a child of 'parent', with the
 *  name 'name' and widget flags set to 'f'
 */

hk_kdetable::hk_kdetable(QWidget* w,const char* n,WFlags f):QWidget(w,n,f),hk_dsvisible()
{
#ifdef HK_DEBUG
    hkdebug("hk_kdetable::hk_kdetable");
    hkclassname("hk_kdetable");
#endif
    setName( "hk_kdetable" );
    resize( 596, 480 );
    //setCaption( tr( ""  ) );
    hk_kdetableLayout = new QVBoxLayout( this );
    hk_kdetableLayout->setSpacing( 0 );
    hk_kdetableLayout->setMargin( 0 );

    toolbar = new hk_kdetoolbar( this, "toolbar" );
    //toolbar->wanna_debug(true);
    toolbar->show_savebutton(false);
    toolbar->set_designmode();
    toolbar->show();
    //toolbar->setTitle( tr( ""  ) );
    hk_kdetableLayout->addWidget( toolbar );
    stack= new QWidgetStack(this);
    stack->setSizePolicy(QSizePolicy(QSizePolicy::Expanding,QSizePolicy::Expanding));
    hk_kdetableLayout->addWidget( stack );
    p_grid = new hk_kdegrid(stack);
    p_grid->set_enablingbehaviour(true,true);
//    p_grid->hkclassname("p_grid");

//    hk_kdetableLayout->addWidget( p_grid );
    p_design = new hk_kdetabledesign(stack);
//    hk_kdetableLayout->addWidget(p_design);
    p_autoclose =true;
    set_nodesignmode(runtime_only());
    designbutton_clicked();
    stack->addWidget(p_design,1);
    stack->addWidget(p_grid,2);

    connect( toolbar, SIGNAL( signal_designmode_selected() ), this, SLOT( designbutton_clicked() ) );
    connect( toolbar, SIGNAL( signal_viewmode_selected() ), this, SLOT( tablebutton_clicked() ) );
    connect( toolbar, SIGNAL( signal_viewmode_selected() ), this, SLOT( tablebutton_clicked() ) );
    connect( toolbar, SIGNAL( signal_searchbutton_clicked() ), p_grid, SLOT( find_clicked() ) );
}

/*
 *  Destroys the object and frees any allocated resources
 */
hk_kdetable::~hk_kdetable()
{
    // no need to delete child widgets, Qt does it all for us
 hk_datasource* d=p_grid->datasource();

 if (d!=NULL) d->disable();

}

void hk_kdetable::set_datasource(hk_datasource* d)
{
  hk_dsvisible::set_datasource(d);
  p_grid->set_datasource(d);
  p_design->set_datasource(d);
  toolbar->set_datasource(d);
  if (d!=NULL)
  {
   //d->enable();
   QString n=hk_translate("Table - ").c_str();
   n+=d->name().c_str();

   setCaption(n);
  }
}

hk_datasource* hk_kdetable::datasource(void)
{

  return hk_dsvisible::datasource();
}

void hk_kdetable::designbutton_clicked(void)
{
   if (p_nodesignmode)
   {
      tablebutton_clicked();
      return;
   }
   hk_datasource* d=p_grid->datasource();
   if (d!=NULL)
   {
      if (d->is_enabled())d->store_changed_data();
      d->disable();
   }
   stack->raiseWidget(1);
   p_designmode=true;
   toolbar->set_designmode();

}


void hk_kdetable::tablebutton_clicked(void)
{
   if (p_designmode &&p_design->datasource()!=NULL)
		       {
			if (!p_design->alter_table())
			{
			 toolbar->set_designmode();
			  return;
			}
		       }
   hk_datasource* d=p_grid->datasource();
   if (d!=NULL) d->enable();
   stack->raiseWidget(2);
   p_designmode=false;
   toolbar->set_viewmode();
//    tablebutton->setOn( TRUE );

}



void hk_kdetable::set_designmode(void)
{
   designbutton_clicked();
}

void hk_kdetable::set_viewmode(void)
{
   tablebutton_clicked();
}

void hk_kdetable::before_source_vanishes(void)
{
#ifdef HK_DEBUG
 hkdebug("hk_kdetable::before_source_vanishes");
#endif

  if (p_autoclose)
    close();
  else
  {
   hk_datasource* d=datasource();
   if (d!=NULL)d->disable();
 if (p_grid!=NULL) p_grid->set_datasource(NULL);
  }
}

void hk_kdetable::closeEvent ( QCloseEvent* e)
{
//   if (   p_designmode) p_design->alter_table();
    QWidget::closeEvent(e);



}

void hk_kdetable::set_nodesignmode(bool d)
{
 p_nodesignmode=d;
 toolbar->set_nodesignmode(d);

}
