// ****************************************************************************
// copyright (c) 2000,2001 Horst Knorr <hk_classes@knorrnet.de>
// This file is part of the hk_kdeclasses library.
// This file may be distributed and/or modified under the terms of the
// GNU General Public License version 2 as published by the Free Software
// Foundation and appearing in the file COPYING included in the
// packaging of this file.
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
// ****************************************************************************
//Version $Revision: 1.10 $

#include "hk_kdetoolbar.h"
#include <qlayout.h>
#include <qtoolbutton.h>
#include <qtooltip.h>
#include <hk_kdefilterdialog.h>
#include <qmultilineedit.h>
#include <hk_datasource.h>


hk_kdetoolbar::hk_kdetoolbar(QWidget* w,const char* s) :hk_kdetoolbarbase(w,s) ,hk_dsvisible()
{
#ifdef HK_DEBUG
hkdebug("hk_kdetoolbar::hk_kdetoolbar");
#endif

  p_filter_should_be_activated=p_is_activated=false;
  p_viewmode=false;
  p_nodesignmode=false;
  show_qbebutton(false);
  show_savebutton(true);
  show_filterbutton(true);
  show_searchbutton(true);
  p_filterexecbutton->setEnabled(false);

  QToolTip::add(  p_designbutton, hk_translate( "Design mode" ).c_str() );
  QToolTip::add(  p_viewbutton, hk_translate( "View mode" ).c_str() );
  QToolTip::add(  p_qbebutton, hk_translate( "Query by Example" ).c_str() );
  QToolTip::add(  p_savebutton, hk_translate( "Store" ).c_str() );
  QToolTip::add(  p_filterdefinitionbutton, hk_translate( "Filter definition" ).c_str() );
  QToolTip::add(  p_filterexecbutton, hk_translate( "Filter execution" ).c_str() );
  QToolTip::add(  p_searchbutton, hk_translate( "Search" ).c_str() );
  QToolTip::add(  p_exitbutton, hk_translate( "Exit" ).c_str() );
  if (runtime_only()) set_nodesignmode(true);
  set_designmode();
}

hk_kdetoolbar::~hk_kdetoolbar(void)
{
#ifdef HK_DEBUG
hkdebug("hk_kdetoolbar::~hk_kdetoolbar");
#endif


}

void hk_kdetoolbar::designbutton_clicked(void)
{
#ifdef HK_DEBUG
hkdebug("hk_kdetoolbar::designbutton_clicked");
#endif
 set_designmode();
 emit signal_designmode_selected();
}

void hk_kdetoolbar::viewbutton_clicked(void)
{
#ifdef HK_DEBUG
hkdebug("hk_kdetoolbar::viewbutton_clicked");
#endif

 set_viewmode();
 emit signal_viewmode_selected();

}


void hk_kdetoolbar::set_designmode(void)
{
#ifdef HK_DEBUG
hkdebug("hk_kdetoolbar::set_designmode");
#endif
 p_viewmode=false;
 p_filterexecbutton->hide();
 p_filterdefinitionbutton->hide();
 p_searchbutton->hide();
    p_designbutton->setOn( TRUE );
    deactivate_filter();
}

void hk_kdetoolbar::set_viewmode(void)
{
#ifdef HK_DEBUG
hkdebug("hk_kdetoolbar::set_viewmode");
#endif
 p_viewmode=true;
 if( p_filter_should_be_activated) activate_filter();
 if (p_showfilterbutton)
 {
    p_filterexecbutton->show();
    p_filterdefinitionbutton->show();
 }

 if (p_showsearchbutton) p_searchbutton->show();

    p_viewbutton->setOn( TRUE );
}

void hk_kdetoolbar::set_nodesignmode(bool d)
{
#ifdef HK_DEBUG
hkdebug("hk_kdetoolbar::set_nodesignmode");
#endif
//  if (d==p_nodesignmode) return;
p_nodesignmode=d;
  if (d)
  {
     p_designbutton->hide();
     p_viewbutton->hide();
     p_qbebutton->hide();
     p_savebutton->hide();
     //modespacer->hide();
  }
  else
  {
     p_designbutton->show();
     p_viewbutton->show();
     //modespacer->show();
     if (p_showqbebutton) p_qbebutton->show();
     if (p_showsavebutton) p_savebutton->show();
  }
}


void hk_kdetoolbar::show_qbebutton(bool s)
{
#ifdef HK_DEBUG
hkdebug("hk_kdetoolbar::show_qbebutton");
#endif
  p_showqbebutton=s;
  if (s && !p_nodesignmode)p_qbebutton->show();
  else p_qbebutton->hide();
}



void hk_kdetoolbar::show_savebutton(bool s)
{
#ifdef HK_DEBUG
hkdebug("hk_kdetoolbar::show_savebutton");
#endif
  p_showsavebutton=s;
  if (s && !p_nodesignmode)p_savebutton->show();
  else p_savebutton->hide();
}



void hk_kdetoolbar::show_filterbutton(bool s)
{
#ifdef HK_DEBUG
hkdebug("hk_kdetoolbar::show_filterbutton");
#endif
  p_showfilterbutton=s;
  if (s)
     {
       p_filterdefinitionbutton->show();
       p_filterexecbutton->show();
     }
  else
     {
       p_filterdefinitionbutton->hide();
       p_filterexecbutton->hide();
     }

}

void hk_kdetoolbar::show_searchbutton(bool s)
{
#ifdef HK_DEBUG
hkdebug("hk_kdetoolbar::show_searchbutton");
#endif
  p_showsearchbutton=s;
  if (s)p_searchbutton->show();
  else p_searchbutton->hide();
}

void hk_kdetoolbar::savebutton_clicked()
{
#ifdef HK_DEBUG
hkdebug("hk_kdetoolbar::savebutton_clicked");
#endif
emit signal_savebutton_clicked();
}


void hk_kdetoolbar::searchbutton_clicked()
{
#ifdef HK_DEBUG
hkdebug("hk_kdetoolbar::searchbutton_clicked");
#endif
emit signal_searchbutton_clicked();
}


void hk_kdetoolbar::exitbutton_clicked()
{
#ifdef HK_DEBUG
hkdebug("hk_kdetoolbar::exitbutton_clicked");
#endif
  if (topLevelWidget()!=NULL) topLevelWidget()->close();

}


void hk_kdetoolbar::filterdefinebutton_clicked()
{
#ifdef HK_DEBUG
hkdebug("hk_kdetoolbar::filterdefinebutton_clicked");
#endif
  hk_kdefilterdialog* f = new hk_kdefilterdialog(this,0,true);
  f->textfield->setText(p_filter);
  if (f->exec()==QDialog::Accepted)
  {
    set_filter(f->textfield->text());
  };

  delete f;
}

void hk_kdetoolbar::set_filter(const QString& s)
{
#ifdef HK_DEBUG
hkdebug("hk_kdetoolbar::set_filter");
#endif
  p_filter=s;
  p_filterexecbutton->setEnabled(!p_filter.isEmpty());

}

void hk_kdetoolbar::filterexecbutton_clicked()
{
#ifdef HK_DEBUG
hkdebug("hk_kdetoolbar::filterexecbutton_clicked");
#endif
    if (datasource()==NULL)return;

    switch (p_filterexecbutton->state())
    {
      case QButton::Off  :    	deactivate_filter();
      				p_filter_should_be_activated=false;
				break;
      case QButton::On  :    	activate_filter();
      				p_filter_should_be_activated=true;
				break;
      default	      :	 	;

    }
}

void hk_kdetoolbar::deactivate_filter(void)
{
#ifdef HK_DEBUG
hkdebug("hk_kdetoolbar::deactivate_filter");
#endif
   p_filterdefinitionbutton->setEnabled(true);
   p_filterexecbutton->blockSignals(true);
   if (p_filterexecbutton->state()!=QButton::Off) p_filterexecbutton->toggle();
   p_filterexecbutton->blockSignals(false);
   if (datasource()==NULL) return;
   if (!p_is_activated)return;
   datasource()->disable();
   datasource()->set_filter(p_originalfilter,false);
   if (p_viewmode)   datasource()->enable();
  p_is_activated=false;

}

void hk_kdetoolbar::activate_filter(void)
{
#ifdef HK_DEBUG
hkdebug("hk_kdetoolbar::activate_filter");
#endif
   p_filterdefinitionbutton->setEnabled(false);
   p_filterexecbutton->blockSignals(true);
   if (p_filterexecbutton->state()!=QButton::On) p_filterexecbutton->toggle();
   p_filterexecbutton->blockSignals(false);
   if (datasource()==NULL) return;

   if (p_is_activated) return;
   p_originalfilter=datasource()->filter();
   hk_string nf;
   if (p_originalfilter.size()>0)
     {
       nf="("+p_originalfilter+") AND (";
     }
   nf+=p_filter.latin1();
   if (p_originalfilter.size()>0) nf+=")";

   //cout <<"neuer Filter: "<<nf<<endl;

   datasource()->disable();
   datasource()->set_filter(nf,false);
   if (p_viewmode) datasource()->enable();
   p_filter_should_be_activated=p_is_activated=true;
}




bool hk_kdetoolbar::is_filter_activated(void)
{
#ifdef HK_DEBUG
hkdebug("hk_kdetoolbar::is_filter_activated");
#endif
  return p_is_activated;
}

