// ****************************************************************************
// copyright (c) 2000,2001 Horst Knorr <hk_classes@knorrnet.de>
// This file is part of knoda -Knorr's Datenbank.
// This file may be distributed and/or modified under the terms of the
// GNU General Public License version 2 as published by the Free Software
// Foundation and appearing in the file COPYING included in the
// packaging of this file.
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
// ****************************************************************************

#include "knodaprogram.h"

#include <qpushbutton.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <kaboutapplication.h>
#include "knodawin.h"
#include <hk_classes.h>
#include "hk_kdepreferencesdialog.h"
#include <qlineedit.h>
#if HAVE_CONFIG_H
#include <config.h>
#endif

/*
 *  Constructs a programwindow which is a child of 'parent', with the
 *  name 'name' and widget flags set to 'f'
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
 */
programwindow::programwindow( QWidget* parent,  const char* name, bool modal, WFlags fl,const hk_string& driver,bool runtime )
    : QDialog( parent, name, modal, fl )
{
    if ( !name )
	setName( "knodaprogramwindow" );

    p_runtime=runtime;
    resize( 175, 120 );
    p_driver=driver;
    setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)3, (QSizePolicy::SizeType)0, sizePolicy().hasHeightForWidth() ) );
    setMinimumSize( QSize( 175, 120 ) );
    setMaximumSize( QSize( 32767, 120 ) );
    QString name="Knoda V ";
    name+=VERSION;
    setCaption( name );
    programformLayout = new QHBoxLayout( this );
    programformLayout->setSpacing( 6 );
    programformLayout->setMargin( 11 );
    QSpacerItem* spacer = new QSpacerItem( 20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum );
    programformLayout->addItem( spacer );

    Layout2 = new QVBoxLayout;
    Layout2->setSpacing( 6 );
    Layout2->setMargin( 0 );

    connectbutton = new QPushButton( this, "connectbutton" );
    connectbutton->setText( hk_class::hk_translate("&Connect"  ).c_str() );
    connectbutton->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)3, (QSizePolicy::SizeType)0, connectbutton->sizePolicy().hasHeightForWidth() ) );
    Layout2->addWidget( connectbutton );

    quitbutton = new QPushButton( this, "quitbutton" );
    quitbutton->setText( hk_class::hk_translate("&Quit"  ).c_str() );
    quitbutton->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)3, (QSizePolicy::SizeType)0, quitbutton->sizePolicy().hasHeightForWidth() ) );
    Layout2->addWidget( quitbutton );

    optionbutton = new QPushButton( this, "optionbutton" );
    optionbutton->setText( hk_class::hk_translate("&Options"  ).c_str() );
    optionbutton->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)3, (QSizePolicy::SizeType)0, optionbutton->sizePolicy().hasHeightForWidth() ) );
    optionbutton->setEnabled(true);
    Layout2->addWidget( optionbutton );

    helpbutton = new QPushButton( this, "helpbutton" );
    helpbutton->setText( hk_class::hk_translate("&Help"  ).c_str() );
    helpbutton->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)3, (QSizePolicy::SizeType)0, helpbutton->sizePolicy().hasHeightForWidth() ) );
    helpbutton->setEnabled(false);
    Layout2->addWidget( helpbutton );
    helpbutton->hide();

    aboutbutton = new QPushButton( this, "aboutbutton" );
    aboutbutton->setText( hk_class::hk_translate("&About"  ).c_str() );
    aboutbutton->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)3, (QSizePolicy::SizeType)0, helpbutton->sizePolicy().hasHeightForWidth() ) );
    aboutbutton->setEnabled(true);
    Layout2->addWidget( aboutbutton );
    programformLayout->addLayout( Layout2 );
    QSpacerItem* spacer_2 = new QSpacerItem( 20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum );
    programformLayout->addItem( spacer_2 );
     p_drivermanager =new hk_drivermanager(p_runtime);
//    p_drivermanager->set_generaldebug(true);

    // signals and slots connections
    connect( connectbutton, SIGNAL( clicked() ), this, SLOT( connect_clicked() ) );
    connect( quitbutton, SIGNAL( clicked() ), this, SLOT( quit_clicked() ) );
    connect( optionbutton, SIGNAL( clicked() ), this, SLOT( option_clicked() ) );
    connect( helpbutton, SIGNAL( clicked() ), this, SLOT( help_clicked() ) );
    connect( aboutbutton, SIGNAL( clicked() ), this, SLOT( about_clicked() ) );
}

/*
 *  Destroys the object and frees any allocated resources
 */
programwindow::~programwindow()
{
    // no need to delete child widgets, Qt does it all for us
delete p_drivermanager;
}

void programwindow::connect_clicked()
{


  hk_connection* c=p_drivermanager->new_connection(p_driver);
  if (c!=NULL)
  {
    if (c->connect())
    {
       knodamainwindow* w=new knodamainwindow(0,0,WDestructiveClose);
       w->set_connection(c);
       w->show();

    }
    else
    {
     delete c;
     c=NULL;
    }
  } else
    {
     p_driver="";
    }
}

void programwindow::help_clicked()
{
    qWarning( "programwindow::help_clicked(): Not implemented yet!" );
}

void programwindow::option_clicked()
{
   hk_kdepreferencesdialog* p_dialog= new hk_kdepreferencesdialog(0,0,true);
   p_dialog->timefield->setText(hk_class::defaulttimeformat().c_str()),
   p_dialog->datefield->setText(hk_class::defaultdateformat().c_str()),
   p_dialog->datetimefield->setText(hk_class::defaultdatetimeformat().c_str());
      int res=p_dialog->exec();
   if (res==QDialog::Accepted)
     {
       hk_class::set_defaultdatetimeformat(
					   p_dialog->timefield->text().latin1(),
					   p_dialog->datefield->text().latin1(),
					   p_dialog->datetimefield->text().latin1()
       					   );
     hk_class::save_preferences();
     }



   delete p_dialog;
}

void programwindow::quit_clicked()
{
   close();
}

void programwindow::about_clicked()
{
  KAboutApplication* d=new KAboutApplication();
  d->show();
  delete d;
}

