// ****************************************************************************
// copyright (c) 2000,2001 Horst Knorr <hk_classes@knorrnet.de>
// This file is part of knoda -Knorr's Datenbank.
// This file may be distributed and/or modified under the terms of the
// GNU General Public License version 2 as published by the Free Software
// Foundation and appearing in the file COPYING included in the
// packaging of this file.
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
// ****************************************************************************

#include "knodawin.h"
#include "knodanewdatabasedialog.h"
#include <qbuttongroup.h>
#include <qlabel.h>
#include <qlistbox.h>
#include <qpushbutton.h>
#include <qradiobutton.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <vector>
#include <hk_class.h>
#include <hk_string.h>
#include <hk_kdetable.h>
#include <hk_kdequery.h>
#include <hk_kdegrid.h>
#include <hk_kdesimplegrid.h>
#include <klocale.h>
#include <hk_kdecsvimportdialog.h>
#include <hk_kdecsvexportdialog.h>
#include <hk_kdeform.h>
/*
 *  Constructs a knodamainwindow which is a child of 'parent', with the
 *  name 'name' and widget flags set to 'f'
 */

class hk_listbox : public QListBox,public hk_class
{
  friend class knodamainwindow;
 public:
  hk_listbox(knodamainwindow* w);
  ~hk_listbox(){}
 protected:
virtual void mousePressEvent(QMouseEvent* m);
    QPopupMenu* p_popup;
    knodamainwindow* p_window;
} ;

 enum enum_functions {fdatabase,ftable,fquery,fform,freport};
 enum enum_menupoint {pm_new,pm_alter,pm_delete,pm_start,pm_import,pm_export,pm_importcsv,pm_exportcsv,pm_importxml,pm_exportxml,pm_exporthtml,pm_copy};

knodamainwindow::knodamainwindow( QWidget* parent,  const char* name, WFlags fl )
    : QWidget( parent, name, fl ) ,hk_dsvisible()
{
#ifdef HK_DEBUG
hkclassname("knodawin");
hkdebug("knodamainwindow::knodamainwindow");
#endif
    if ( !name )
	setName( "knodamainwindow" );
    resize( 396, 422 );
    setCaption(  "Knoda - Knorr's Datenbank"   );
    setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)3, (QSizePolicy::SizeType)3, sizePolicy().hasHeightForWidth() ) );
    setMinimumSize( QSize( 396, 421 ) );
    setMaximumSize( QSize( 32767, 32767 ) );
    knodamainwindowLayout = new QHBoxLayout( this );
    knodamainwindowLayout->setSpacing( 6 );
    knodamainwindowLayout->setMargin( 11 );
    p_tablestring=hk_class::hk_translate("Tables").c_str();
    p_querystring=hk_class::hk_translate("Queries").c_str();
    p_formstring=hk_class::hk_translate("Forms").c_str();
    p_reportstring=hk_class::hk_translate("Reports").c_str();
    p_databasestring=hk_class::hk_translate("Databases").c_str();

    Layout8 = new QGridLayout(this);
    Layout8->setSpacing( 6 );
    Layout8->setMargin( 0 );

    itemslist = new hk_listbox( this );
    itemslist->setMinimumSize( QSize( 250, 300 ) );
    itemslist->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)3, (QSizePolicy::SizeType)3, itemslist->sizePolicy().hasHeightForWidth() ) );

    Layout8->addWidget( itemslist, 1, 0 );

    TextLabel1 = new QLabel( this, "TextLabel1" );
    TextLabel1->setText( p_databasestring );

    Layout8->addWidget( TextLabel1, 0, 0 );
    knodamainwindowLayout->addLayout( Layout8 );

    Layout3 = new QVBoxLayout;
    Layout3->setSpacing( 6 );
    Layout3->setMargin( 0 );

    TextLabel1_2 = new QLabel( this, "TextLabel1_2" );
    TextLabel1_2->setText( hk_class::hk_translate( "Actual database:"  ).c_str() );
    Layout3->addWidget( TextLabel1_2 );

    databasename = new QLabel( this, "databasename" );
    databasename->setAlignment( int( QLabel::WordBreak | QLabel::AlignCenter ) );
    databasename->setFrameShape( QLabel::Box );
    databasename->setFrameShadow( QLabel::Sunken );
    databasename->setScaledContents( TRUE );
    databasename->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)5, (QSizePolicy::SizeType)7, databasename->sizePolicy().hasHeightForWidth() ) );
    databasename->setMinimumSize( QSize( 0, 30 ) );
    Layout3->addWidget( databasename );
    ButtonGroup2 = new QButtonGroup( this, "ButtonGroup2" );
    ButtonGroup2->setTitle( hk_class::hk_translate( "Functions"  ).c_str() );
    ButtonGroup2->setExclusive( TRUE );
    ButtonGroup2->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)5, (QSizePolicy::SizeType)0, ButtonGroup2->sizePolicy().hasHeightForWidth() ) );
    ButtonGroup2->setMinimumSize( QSize( 0, 157 ) );
    ButtonGroup2->setColumnLayout(0, Qt::Vertical );
    ButtonGroup2->layout()->setSpacing( 0 );
    ButtonGroup2->layout()->setMargin( 0 );

    ButtonGroup2Layout = new QVBoxLayout( ButtonGroup2->layout() );
    ButtonGroup2Layout->setAlignment( Qt::AlignTop );
    ButtonGroup2Layout->setSpacing( 6 );
    ButtonGroup2Layout->setMargin( 11 );

    querybutton = new QRadioButton( ButtonGroup2, "querybutton" );
    querybutton->setGeometry( QRect( 11, 71, 73, 19 ) );
    querybutton->setText(  p_querystring );
    ButtonGroup2->insert( querybutton, 2 );
    querybutton->setEnabled( FALSE );
    querybutton->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)1, (QSizePolicy::SizeType)0, querybutton->sizePolicy().hasHeightForWidth() ) );

    reportbutton = new QRadioButton( ButtonGroup2, "reportbutton" );
    reportbutton->setGeometry( QRect( 11, 121, 73, 19 ) );
    reportbutton->setText(  p_reportstring );
    ButtonGroup2->insert( reportbutton, 4 );
    reportbutton->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)1, (QSizePolicy::SizeType)0, reportbutton->sizePolicy().hasHeightForWidth() ) );
    reportbutton->setEnabled( FALSE );

    tablebutton = new QRadioButton( ButtonGroup2, "tablebutton" );
    tablebutton->setGeometry( QRect( 11, 46, 73, 19 ) );
    tablebutton->setText( p_tablestring );
    ButtonGroup2->insert( tablebutton, 1 );
    tablebutton->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)1, (QSizePolicy::SizeType)0, tablebutton->sizePolicy().hasHeightForWidth() ) );
    tablebutton->setEnabled( FALSE );

    formbutton = new QRadioButton( ButtonGroup2, "formbutton" );
    formbutton->setGeometry( QRect( 11, 96, 73, 19 ) );
    formbutton->setText( p_formstring );
    ButtonGroup2->insert( formbutton, 3 );
    formbutton->setEnabled( FALSE );

    databasebutton = new QRadioButton( ButtonGroup2, "databasebutton" );
    databasebutton->setGeometry( QRect( 11, 21, 73, 19 ) );
    databasebutton->setText( p_databasestring );
    databasebutton->setChecked( TRUE );
    ButtonGroup2->insert( databasebutton, 0 );
    databasebutton->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)1, (QSizePolicy::SizeType)0, databasebutton->sizePolicy().hasHeightForWidth() ) );
    databasebutton->setEnabled( FALSE );
    ButtonGroup2Layout->addWidget( databasebutton );
    ButtonGroup2Layout->addWidget( tablebutton );
    ButtonGroup2Layout->addWidget( querybutton );
    ButtonGroup2Layout->addWidget( formbutton );
    ButtonGroup2Layout->addWidget( reportbutton );
    Layout3->addWidget( ButtonGroup2 );

    ButtonGroup3 = new QButtonGroup( this, "ButtonGroup3" );
    ButtonGroup3->setTitle( hk_class::hk_translate( "Actions"  ).c_str() );
    ButtonGroup3->setMinimumSize( QSize( 0, 170 ) );
    ButtonGroup3->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)5, (QSizePolicy::SizeType)0, ButtonGroup3->sizePolicy().hasHeightForWidth() ) );
    ButtonGroup3->setColumnLayout(0, Qt::Vertical );
    ButtonGroup3->layout()->setSpacing( 0 );
    ButtonGroup3->layout()->setMargin( 0 );
    ButtonGroup3Layout = new QVBoxLayout( ButtonGroup3->layout() );
    ButtonGroup3Layout->setAlignment( Qt::AlignTop );
    ButtonGroup3Layout->setSpacing( 6 );
    ButtonGroup3Layout->setMargin( 11 );

    newbutton = new QPushButton( ButtonGroup3, "newbutton" );
    newbutton->setText( hk_class::hk_translate( "New"  ).c_str() );
    newbutton->setEnabled( true );
    newbutton->setMinimumSize( QSize( 0, 20 ) );
    newbutton->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)1, (QSizePolicy::SizeType)0, newbutton->sizePolicy().hasHeightForWidth() ) );
    ButtonGroup3Layout->addWidget( newbutton );

    startbutton = new QPushButton( ButtonGroup3, "startbutton" );
    startbutton->setText( hk_class::hk_translate( "Start"  ).c_str() );
    startbutton->setEnabled( FALSE );
    startbutton->setMinimumSize( QSize( 0, 20 ) );
    startbutton->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)1, (QSizePolicy::SizeType)0, startbutton->sizePolicy().hasHeightForWidth() ) );
    ButtonGroup3Layout->addWidget( startbutton );

    alterbutton = new QPushButton( ButtonGroup3, "alterbutton" );
    alterbutton->setText( hk_class::hk_translate( "Alter"  ).c_str() );
    alterbutton->setEnabled( FALSE );
    ButtonGroup3Layout->addWidget( alterbutton );

    deletebutton = new QPushButton( ButtonGroup3, "deletebutton" );
    deletebutton->setText( hk_class::hk_translate( "Delete"  ).c_str() );
    deletebutton->setEnabled( FALSE );
    ButtonGroup3Layout->addWidget( deletebutton );
    Layout3->addWidget( ButtonGroup3 );
    knodamainwindowLayout->addLayout( Layout3 );

    // signals and slots connections
    connect( newbutton, SIGNAL( clicked() ), this, SLOT( new_clicked() ) );
    connect( startbutton, SIGNAL( clicked() ), this, SLOT( start_clicked() ) );
    connect( alterbutton, SIGNAL( clicked() ), this, SLOT( alter_clicked() ) );
    connect( deletebutton, SIGNAL( clicked() ), this, SLOT( delete_clicked() ) );
    connect( itemslist, SIGNAL( doubleClicked(QListBoxItem*) ), this, SLOT( itemslist_doubleclicked() ) );
    connect( itemslist, SIGNAL( clicked(QListBoxItem*) ), this, SLOT( itemslist_clicked(QListBoxItem*) ) );
    connect( ButtonGroup2, SIGNAL( clicked(int) ), this, SLOT( function_select(int) ) );
    p_connection=NULL;
    p_database=NULL;
    if (runtime_only())
       {
          alterbutton->hide();
	  newbutton->hide();
          deletebutton->hide();
       }


}

/*
 *  Destroys the object and frees any allocated resources
 */
knodamainwindow::~knodamainwindow()
{

//set_generaldebug(true);

#ifdef HK_DEBUG
hkdebug("=========================>>>>>>>>>>>>>>>");
hkdebug("knodamainwindow::~knodamainwindow");
#endif
    // no need to delete child widgets, Qt does it all for us
    if (p_connection!=NULL)
    {
//     p_connection->new_database();

    p_connection->disconnect();
      delete p_connection;
      p_connection=NULL;
      p_database=NULL;
    }

}
void knodamainwindow::set_connection(hk_connection*c )
{
  p_connection=c;
//  if (p_connection!=NULL) p_connection->wanna_debug(true);
  function_select(fdatabase);
  databasebutton->setEnabled(true);
}


void knodamainwindow::importcsv_clicked(void)
{
#ifdef HK_DEBUG
hkdebug("knodamainwindow::importcsv_clicked");
#endif
   if (p_database==NULL) return;
   hk_kdecsvimportdialog* i= new hk_kdecsvimportdialog(0,0,true);
   hk_datasource* ds=p_database->new_table();
//   ds->wanna_debug(true);
   i->set_datasource(ds);
   i->exec();
   delete i;
}

void knodamainwindow::exportcsv_clicked(void)
{
#ifdef HK_DEBUG
hkdebug("knodamainwindow::exportcsv_clicked");
#endif
   if (p_database==NULL) return;
   hk_kdecsvexportdialog* e =new hk_kdecsvexportdialog(0,0,true);
   e->set_database(p_database);
   e->set_usetablelist(tablebutton->isChecked());
   e->exec();
   delete e;
}

void knodamainwindow::alter_clicked()
{
#ifdef HK_DEBUG
hkdebug("knodamainwindow::alter_clicked");
#endif
   if (p_database==NULL) return;
  int item=itemslist->currentItem();
  if (item==-1)return;
  if (tablebutton->isChecked())
  {
   //hk_kdetable ffnen


   hk_kdetable* w=new hk_kdetable(0,0,WDestructiveClose);
   hk_datasource* d=p_database->new_table();
//   d->wanna_debug(true);
   d->set_name(itemslist->text(itemslist->currentItem()).ascii());
   w->set_datasource(d);
   w->set_designmode();
   w->show();
  return ;
  }

  else
  if (querybutton->isChecked())
  {
   hk_kdequery* w=new hk_kdequery();
   hk_datasource* d=p_database->new_resultquery();
   w->set_datasource(d);
//   d->wanna_debug(true);
   d->set_name(itemslist->text(itemslist->currentItem()).ascii());
   w->load_query();
//   d->sql(p_database->load(itemslist->text(itemslist->currentItem()).ascii(),ft_query).c_str());
   w->set_sqlmode();
   w->show();
  return ;
  }
    else
  if (formbutton->isChecked())
  {
//    set_generaldebug(true);
    hk_kdeform* w=new hk_kdeform(0,0,WDestructiveClose);
    w->set_database(p_database);
    w->load_form(itemslist->text(itemslist->currentItem()).ascii());
    w->show();
    w->set_designmode();

  }
  else
    qWarning( "knodamainwindow::alter_clicked(): Not implemented yet!" );
}

void knodamainwindow::delete_clicked()
{
#ifdef HK_DEBUG
hkdebug("knodamainwindow::delete_clicked");
#endif
    if (databasebutton->isChecked())
    {
      if (itemslist->currentItem()>-1)
      if (p_connection->delete_database(itemslist->text(itemslist->currentItem()).ascii()))
      {
       function_select(fdatabase);
       databasename->setText("");
       set_buttons();
      }
    }
    else
    if (tablebutton->isChecked())
    {
   if (p_database==NULL) return;
      if (itemslist->currentItem()>-1)
      if (p_database->delete_table(itemslist->text(itemslist->currentItem()).ascii()))
      {
       function_select(ftable);
      }
    }
    else
    {
   if (p_database==NULL) return;
      filetype f=ft_query;
      if (formbutton->isChecked()) f=ft_form;
      else
      if (reportbutton->isChecked()) f=ft_report;

     p_database->delete_file(itemslist->text(itemslist->currentItem()).ascii(),f);
    }
}

void knodamainwindow::function_select(int f)
{
#ifdef HK_DEBUG
hkdebug("knodamainwindow::function_select");
#endif
 if (p_connection==NULL) return;
 itemslist->clear();
 switch (f)
 {
      case fdatabase:
    			{
    			TextLabel1->setText( p_databasestring );
			if (!p_connection->is_connected())p_connection->connect();
			const  vector<hk_string>* db=p_connection->dblist();
     			vector<hk_string>::iterator it;
   			if (db!=NULL)
    			 {
   			  for(it=(hk_string*)db->begin();it!=db->end();it++)
     			   {
      				itemslist->insertItem((*it).c_str());
     			   }
   			 }

                         }
			break;

      case ftable:     {
			    if (p_database==NULL) break;
			    TextLabel1->setText( p_tablestring );

			      vector<hk_string>* tb=p_database->tablelist();
     			vector<hk_string>::iterator it;
   			if (tb!=NULL)
    			 {
   			  for(it=(hk_string*)tb->begin();it!=tb->end();it++)
     			   {
      				itemslist->insertItem((*it).c_str());
     			   }
   			 }
      			}
      			break;

      case fquery:       {
			    if (p_database==NULL) break;
			    TextLabel1->setText( p_querystring );
			      vector<hk_string>* tb=p_database->querylist();
     			   vector<hk_string>::iterator it;
   			   if (tb!=NULL)
    			 {
   			  for(it=(hk_string*)tb->begin();it!=tb->end();it++)
     			   {
      				itemslist->insertItem((*it).c_str());
     			   }
   			 }
			}
      			break;

      case fform:         {
			    if (p_database==NULL) break;
			    TextLabel1->setText( p_formstring );
			      vector<hk_string>* tb=p_database->formlist();
     			   vector<hk_string>::iterator it;
   			   if (tb!=NULL)
    			 {
   			  for(it=(hk_string*)tb->begin();it!=tb->end();it++)
     			   {
      				itemslist->insertItem((*it).c_str());
     			   }
   			 }
			 }
      			break;

      case freport:
			    if (p_database==NULL) break;
			    TextLabel1->setText( p_reportstring );
      			break;

      default:;

 }
  itemslist->setCurrentItem(-1);

 set_buttons();
 itemslist->repaint(true);
}

void knodamainwindow::itemslist_doubleclicked()
{
#ifdef HK_DEBUG
hkdebug("knodamainwindow::itemslist_doubleclicked");
#endif
 start_clicked();
}

void knodamainwindow::new_clicked()
{
 #ifdef HK_DEBUG
hkdebug("knodamainwindow::new_clicked");
#endif

    if (databasebutton->isChecked())
    {

     knodanewdatabasedialog* d =new knodanewdatabasedialog(0,0,true);
     int r=d->exec();
     if (r==QDialog::Accepted)
      {
	p_connection->create_database(d->databasename->text().ascii());
	function_select(fdatabase);
      }
//     delete d;
    }
    else
  if (tablebutton->isChecked())
  {
   if (p_database==NULL) return;
   hk_kdetable* t=new hk_kdetable();
   hk_datasource* d=p_database->new_table();
   t->set_datasource(d);
   t->set_designmode();
   t->show();
  }
  else
  if (querybutton->isChecked())
  {
   if (p_database==NULL) return;
   hk_kdequery* w=new hk_kdequery();
   hk_datasource* d=p_database->new_resultquery();
   w->set_datasource(d);
   w->set_sqlmode();
   w->show();
  return ;
  }

  else
  if (formbutton->isChecked())
  {
   if (p_database==NULL) return;
   hk_kdeform* w=new hk_kdeform(0,0,WDestructiveClose);
   w->set_database(p_database);
   w->set_designmode();
   w->show();
  return ;
  }
  else    qWarning( "knodamainwindow::new_clicked(): Not implemented yet!" );
}

void knodamainwindow::start_clicked()
{
   if (p_database==NULL) return;
  int item=itemslist->currentItem();
  if (item==-1)return;
  if (tablebutton->isChecked())
  {
   //hk_kdetable ffnen
   hk_kdetable* w=new hk_kdetable(0,0,WDestructiveClose);
   hk_datasource* d=p_database->new_table();
//   d->wanna_debug(true);
   d->set_name(itemslist->text(itemslist->currentItem()).ascii());
   w->set_datasource(d);
   w->show();
   w->set_viewmode();
  return ;
  }

  else
  if (querybutton->isChecked())
  {
   hk_kdequery* w=new hk_kdequery(0,0,WDestructiveClose);
   hk_datasource* d=p_database->new_resultquery();
   d->set_name(itemslist->text(itemslist->currentItem()).ascii());
   w->set_datasource(d);
   w->load_query();
   w->show();
   w->set_viewmode();
  return ;
  }
  else
  if (formbutton->isChecked())
  {
   hk_kdeform* w=new hk_kdeform(0,0,WDestructiveClose);
   w->set_database(p_database);
   w->load_form(itemslist->text(itemslist->currentItem()).ascii());

   w->showMaximized();
   w->set_viewmode();
  return ;
  }
  else

  qWarning( "knodamainwindow::start_clicked(): Not implemented yet!" );
}


void knodamainwindow::itemslist_clicked(QListBoxItem* item)
{
#ifdef HK_DEBUG
hkdebug("knodamainwindow::itemslist_clicked");
#endif

  if (item==NULL)return;
  if (databasebutton->isChecked())
  {

  	if (item->text()!=databasename->text())
  	{
    		p_database=p_connection->new_database();
    		if (p_database!=NULL)
    		{
		  #ifdef HK_DEBUG
		  p_database->hkclassname("knodawindatabase");
		  #endif
     		  p_database->set_name(item->text().ascii());
     		  set_datasource(p_database->new_resultquery());
     		  databasename->setText(item->text());
    		}
  	}
  }

  set_buttons();
}


void	knodamainwindow::list_changes(filetype type)
{
#ifdef HK_DEBUG
hkdebug("knodamainwindow::list_changes");
#endif
   int f;
     switch (type)
      {
       case ft_query : f=fquery;
       			break;
       case ft_form : f=fform;
       			break;
       case ft_report : f=freport;
       			break;
       case ft_table : f=ftable;
       			break;
       default : f=fquery;

      }
     if (type==ft_query && !querybutton->isChecked()) return;
     if (type==ft_form && !formbutton->isChecked()) return;
     if (type==ft_report && !reportbutton->isChecked()) return;
    function_select(f);
    set_buttons();
}


hk_listbox::hk_listbox(knodamainwindow* w):QListBox(w),hk_class()
{
    p_window=w;
    p_popup=new QPopupMenu(this);
    QPopupMenu* p_submenuimport =new QPopupMenu(this);
    QPopupMenu* p_submenuexport =new QPopupMenu(this);
     if (!runtime_only())
       {
     	 p_popup->insertItem( hk_class::hk_translate( "New" ).c_str(),pm_new );
         p_popup->insertItem( hk_class::hk_translate( "Alter" ).c_str(),pm_alter );
         p_popup->insertItem( hk_class::hk_translate( "Delete" ).c_str(),pm_delete );
         p_popup->insertItem( hk_class::hk_translate( "Import" ).c_str(),p_submenuimport ,pm_import);
       }
     p_popup->insertItem( hk_class::hk_translate( "Start" ).c_str(),pm_start );
     p_popup->insertItem( hk_class::hk_translate( "Export" ).c_str(),p_submenuexport,pm_export );
     p_popup->insertItem( hk_class::hk_translate( "Copy" ).c_str(),pm_copy );

     p_submenuimport->insertItem( hk_class::hk_translate( "CSV" ).c_str(),pm_importcsv );
     p_submenuimport->insertItem( hk_class::hk_translate( "XML" ).c_str(),pm_importxml );
     p_submenuexport->insertItem( hk_class::hk_translate( "CSV" ).c_str(),pm_exportcsv );
     p_submenuexport->insertItem( hk_class::hk_translate( "HTML" ).c_str(),pm_exporthtml );
     p_submenuexport->insertItem( hk_class::hk_translate( "XML" ).c_str(),pm_exportxml );
     p_popup->setItemEnabled(pm_alter,false);
     p_popup->setItemEnabled(pm_start,false);
     p_popup->setItemEnabled(pm_delete,false);
     p_popup->setItemEnabled(pm_import,false);
     p_popup->setItemEnabled(pm_export,false);
     p_popup->setItemEnabled(pm_importxml,false);
     p_popup->setItemEnabled(pm_exportxml,false);
     p_popup->setItemEnabled(pm_exporthtml,false);
     p_popup->setItemEnabled(pm_copy,false);
}

void hk_listbox::mousePressEvent(QMouseEvent* m)
{

     QListBox::mousePressEvent(m);

     if (m->button()==RightButton)
     {
     	int r = p_popup->exec( m->globalPos() );
	switch (r)
	{
	 case pm_new:
	 			p_window->new_clicked();
	 			break;
	 case pm_alter:
	 			p_window->alter_clicked();
	 			break;
	 case pm_start:
	 			p_window->start_clicked();
	 			break;
	 case pm_delete:
	 			p_window->delete_clicked();
	 			break;
	 case pm_importcsv:
				p_window->importcsv_clicked();
				break;
	 case pm_exportcsv:
				p_window->exportcsv_clicked();
				break;
	 default:
				cerr <<"Not implemented yet"<<endl;
				;
	}

     }
}

void knodamainwindow::set_buttons()
{
#ifdef HK_DEBUG
hkdebug("knodamainwindow::set_buttons");
#endif

  if (itemslist->currentItem()>-1) set_normalbuttons();
    else   set_minimalbuttons();

  if (databasebutton->isChecked()) set_databasebuttons();
  else
  if (tablebutton->isChecked()) set_tablebuttons();
  else
  if (formbutton->isChecked()) set_formbuttons();
  else
  if (querybutton->isChecked()) set_querybuttons();
  else
  if (reportbutton->isChecked()) set_reportbuttons();



}

void knodamainwindow::set_databasebuttons()
{
 #ifdef HK_DEBUG
hkdebug("knodamainwindow::set_databasebuttons");
#endif
  startbutton->setEnabled(false);
   alterbutton->setEnabled(false);
   itemslist->p_popup->setItemEnabled(pm_start,false);
   itemslist->p_popup->setItemEnabled(pm_alter,false);
}

void knodamainwindow::set_tablebuttons()
{
#ifdef HK_DEBUG
hkdebug("knodamainwindow::set_tablebuttons");
#endif
   itemslist->p_popup->setItemEnabled(pm_import,true);
   if(itemslist->currentItem()>-1)
      itemslist->p_popup->setItemEnabled(pm_export,true);
}

void knodamainwindow::set_querybuttons()
{
#ifdef HK_DEBUG
hkdebug("knodamainwindow::set_querybuttons");
#endif
   if(itemslist->currentItem()>-1)
      itemslist->p_popup->setItemEnabled(pm_export,true);

}

void knodamainwindow::set_formbuttons()
{
#ifdef HK_DEBUG
hkdebug("knodamainwindow::set_formbuttons");
#endif
//   newbutton->setEnabled(false);
//   itemslist->p_popup->setItemEnabled(pm_new,false);
}

void knodamainwindow::set_reportbuttons()
{
#ifdef HK_DEBUG
hkdebug("knodamainwindow::set_reportbuttons");
#endif
   newbutton->setEnabled(false);
   itemslist->p_popup->setItemEnabled(pm_new,false);
}

void knodamainwindow::set_minimalbuttons()
{
#ifdef HK_DEBUG
hkdebug("knodamainwindow::set_minimalbuttons");
#endif
   newbutton->setEnabled(true);
   startbutton->setEnabled(false);
   alterbutton->setEnabled(false);
   deletebutton->setEnabled(false);
   itemslist->p_popup->setItemEnabled(pm_new,true);
   itemslist->p_popup->setItemEnabled(pm_start,false);
   itemslist->p_popup->setItemEnabled(pm_alter,false);
   itemslist->p_popup->setItemEnabled(pm_delete,false);
   itemslist->p_popup->setItemEnabled(pm_copy,false);
   itemslist->p_popup->setItemEnabled(pm_import,false);
   itemslist->p_popup->setItemEnabled(pm_export,false);
   if (databasename->text().isEmpty())
     {
       tablebutton->setEnabled(false);
       querybutton->setEnabled(false);
       formbutton->setEnabled(false);
       reportbutton->setEnabled(false);
     }
   else
   {
       tablebutton->setEnabled(true);
       querybutton->setEnabled(true);
       formbutton->setEnabled(true);
       reportbutton->setEnabled(true);
   }

}

void knodamainwindow::set_normalbuttons()
{
//	cout<<"itemslist->currentItem() "<<   itemslist->currentItem() <<endl;
#ifdef HK_DEBUG
hkdebug("knodamainwindow::set_normalbuttons");
#endif
   set_minimalbuttons();
   newbutton->setEnabled(true);
   startbutton->setEnabled(true);
   alterbutton->setEnabled(true);
   deletebutton->setEnabled(true);
   itemslist->p_popup->setItemEnabled(pm_start,true);
   itemslist->p_popup->setItemEnabled(pm_alter,true);
   itemslist->p_popup->setItemEnabled(pm_delete,true);
   itemslist->p_popup->setItemEnabled(pm_copy,true);
}

